@@ -13,11 +13,26 @@ module Sequel
         ::Informix.connect(opts[:database], opts[:user], opts[:password])
       end
     
+      def transaction(opts=OPTS)
+        if @opts[:nolog]
+          yield
+        else
+          super
+        end
+      end
+
       # Returns number of rows affected
       def execute_dui(sql, opts=OPTS)
         synchronize(opts[:server]){|c| log_yield(sql){c.immediate(sql)}}
       end
       
+      def execute_insert(sql, opts=OPTS)
+        synchronize(opts[:server]){|c|
+          log_yield(sql){c.immediate(sql)}
+          c.cursor(%q{select first 1 dbinfo('sqlca.sqlerrd1') from systables}).open.fetch
+        }
+      end
+
       def execute(sql, opts=OPTS)
         synchronize(opts[:server]){|c| yield log_yield(sql){c.cursor(sql)}}
       end
