@@ -1,11 +1,21 @@
+Sequel::JDBC.load_driver('org.apache.derby.jdbc.EmbeddedDriver', :Derby)
 Sequel.require 'adapters/jdbc/transactions'
 
 module Sequel
   module JDBC
+    Sequel.synchronize do
+      DATABASE_SETUP[:derby] = proc do |db|
+        db.extend(Sequel::JDBC::Derby::DatabaseMethods)
+        db.dataset_class = Sequel::JDBC::Derby::Dataset
+        org.apache.derby.jdbc.EmbeddedDriver
+      end
+    end
+
     # Database and Dataset support for Derby databases accessed via JDBC.
     module Derby
       # Instance methods for Derby Database objects accessed via JDBC.
       module DatabaseMethods
+        extend Sequel::Database::ResetIdentifierMangling
         PRIMARY_KEY_INDEX_RE = /\Asql\d+\z/i.freeze
 
         include ::Sequel::JDBC::Transactions
@@ -37,6 +47,11 @@ module Sequel
           end
         end
         
+        # Derby supports transaction DDL statements.
+        def supports_transactional_ddl?
+          true
+        end
+
         private
         
         # Derby optimizes away Sequel's default check of SELECT NULL FROM table,
@@ -70,17 +85,48 @@ module Sequel
           sql << " NOT NULL" if column.fetch(:null, column[:allow_null]) == false
         end
     
-        # Temporary table creation on Derby use DECLARE instead of CREATE.
+        # Add NOT LOGGED for temporary tables to improve performance.
         def create_table_sql(name, generator, options)
+          s = super
+          s << ' NOT LOGGED' if options[:temp]
+          s
+        end
+
+        # Insert data from the current table into the new table after
+        # creating the table, since it is not possible to do it in one step.
+        def create_table_as(name, sql, options)
+          super
+          from(name).insert(sql.is_a?(Dataset) ? sql : dataset.with_sql(sql))
+        end
+
+        # Derby currently only requires WITH NO DATA, with a separate insert
+        # to import data.
+        def create_table_as_sql(name, sql, options)
+          "#{create_table_prefix_sql(name, options)} AS #{sql} WITH NO DATA"
+        end
+
+        # Temporary table creation on Derby uses DECLARE instead of CREATE.
+        def create_table_prefix_sql(name, options)
           if options[:temp]
-            "DECLARE GLOBAL TEMPORARY TABLE #{quote_identifier(name)} (#{column_list_sql(generator)}) NOT LOGGED"
+            "DECLARE GLOBAL TEMPORARY TABLE #{quote_identifier(name)}"
           else
             super
           end
         end
 
+        DATABASE_ERROR_REGEXPS = {
+          /The statement was aborted because it would have caused a duplicate key value in a unique or primary key constraint or unique index/ => UniqueConstraintViolation,
+          /violation of foreign key constraint/ => ForeignKeyConstraintViolation,
+          /The check constraint .+ was violated/ => CheckConstraintViolation,
+          /cannot accept a NULL value/ => NotNullConstraintViolation,
+          /A lock could not be obtained due to a deadlock/ => SerializationFailure,
+        }.freeze
+        def database_error_regexps
+          DATABASE_ERROR_REGEXPS
+        end
+
         # Use IDENTITY_VAL_LOCAL() to get the last inserted id.
-        def last_insert_id(conn, opts={})
+        def last_insert_id(conn, opts=OPTS)
           statement(conn) do |stmt|
             sql = 'SELECT IDENTITY_VAL_LOCAL() FROM sysibm.sysdummy1'
             rs = log_yield(sql){stmt.executeQuery(sql)}
@@ -89,6 +135,11 @@ module Sequel
           end
         end
 
+        # Handle nil values by using setNull with the correct parameter type.
+        def set_ps_arg_nil(cps, i)
+          cps.setNull(i, cps.getParameterMetaData.getParameterType(i))
+        end
+      
         # Derby uses RENAME TABLE syntax to rename tables.
         def rename_table_sql(name, new_name)
           "RENAME TABLE #{quote_schema_table(name)} TO #{quote_schema_table(new_name)}"
@@ -113,6 +164,16 @@ module Sequel
             super
           end
         end
+
+        # Derby uses clob for text types.
+        def uses_clob_for_text?
+          true
+        end
+
+        # The SQL query to issue to check if a connection is valid.
+        def valid_connection_sql
+          @valid_connection_sql ||= select(1).sql
+        end
       end
       
       # Dataset class for Derby datasets accessed via JDBC.
@@ -121,8 +182,6 @@ module Sequel
         PAREN_OPEN = Dataset::PAREN_OPEN
         OFFSET = Dataset::OFFSET
         CAST_STRING_OPEN = "RTRIM(".freeze
-        BITCOMP_OPEN = "((0 - ".freeze
-        BITCOMP_CLOSE = ") - 1)".freeze
         BLOB_OPEN = "CAST(X'".freeze
         BLOB_CLOSE = "' AS BLOB)".freeze
         HSTAR = "H*".freeze
@@ -131,20 +190,17 @@ module Sequel
         ROWS = " ROWS".freeze
         FETCH_FIRST = " FETCH FIRST ".freeze
         ROWS_ONLY = " ROWS ONLY".freeze
-        BOOL_TRUE = '(1 = 1)'.freeze
-        BOOL_FALSE = '(1 = 0)'.freeze
-        SELECT_CLAUSE_METHODS = clause_methods(:select, %w'select distinct columns from join where group having compounds order limit lock')
+        BOOL_TRUE_OLD = '(1 = 1)'.freeze
+        BOOL_FALSE_OLD = '(1 = 0)'.freeze
+        BOOL_TRUE = 'TRUE'.freeze
+        BOOL_FALSE = 'FALSE'.freeze
+        EMULATED_FUNCTION_MAP = {:char_length=>'length'.freeze}
 
         # Derby doesn't support an expression between CASE and WHEN,
-        # so emulate it by using an equality statement for all of the
+        # so remove 
         # conditions.
         def case_expression_sql_append(sql, ce)
-          if ce.expression?
-            e = ce.expression
-            case_expression_sql_append(sql, ::Sequel::SQL::CaseExpression.new(ce.conditions.map{|c, r| [::Sequel::SQL::BooleanExpression.new(:'=', e, c), r]}, ce.default))
-          else
-            super
-          end
+          super(sql, ce.with_merged_expression)
         end
 
         # If the type is String, trim the extra spaces since CHAR is used instead
@@ -160,19 +216,12 @@ module Sequel
           end
         end
 
-        # Handle Derby specific LIKE, extract, and some bitwise compliment support.
         def complex_expression_sql_append(sql, op, args)
           case op
-          when :ILIKE
-            super(sql, :LIKE, [SQL::Function.new(:upper, args.at(0)), SQL::Function.new(:upper, args.at(1))])
-          when :"NOT ILIKE"
-            super(sql, :"NOT LIKE", [SQL::Function.new(:upper, args.at(0)), SQL::Function.new(:upper, args.at(1))])
+          when :%, :'B~'
+            complex_expression_emulate_append(sql, op, args)
           when :&, :|, :^, :<<, :>>
             raise Error, "Derby doesn't support the #{op} operator"
-          when :'B~'
-            sql << BITCOMP_OPEN
-            literal_append(sql, args.at(0))
-            sql << BITCOMP_CLOSE
           when :extract
             sql << args.at(0).to_s << PAREN_OPEN
             literal_append(sql, args.at(1))
@@ -199,6 +248,10 @@ module Sequel
 
         private
 
+        def empty_from_sql
+          DEFAULT_FROM
+        end
+
         # Derby needs a hex string casted to BLOB for blobs.
         def literal_blob_append(sql, v)
           sql << BLOB_OPEN << v.unpack(HSTAR).first << BLOB_CLOSE
@@ -213,7 +266,11 @@ module Sequel
         # Derby uses an expression yielding false for false values.
         # Newer versions can use the FALSE literal, but the latest gem version cannot.
         def literal_false
-          BOOL_FALSE
+          if db.svn_version >= 1040133
+            BOOL_FALSE
+          else
+            BOOL_FALSE_OLD
+          end
         end
 
         # Derby handles fractional seconds in timestamps, but not in times
@@ -224,23 +281,18 @@ module Sequel
         # Derby uses an expression yielding true for true values.
         # Newer versions can use the TRUE literal, but the latest gem version cannot.
         def literal_true
-          BOOL_TRUE
-        end
-
-        # Derby doesn't support common table expressions.
-        def select_clause_methods
-          SELECT_CLAUSE_METHODS
-        end
-
-        # Use a default FROM table if the dataset does not contain a FROM table.
-        def select_from_sql(sql)
-          if @opts[:from]
-            super
+          if db.svn_version >= 1040133
+            BOOL_TRUE
           else
-            sql << DEFAULT_FROM
+            BOOL_TRUE_OLD
           end
         end
 
+        # Derby supports multiple rows in INSERT.
+        def multi_insert_sql_strategy
+          :values
+        end
+
         # Offset comes before limit in Derby
         def select_limit_sql(sql)
           if o = @opts[:offset]
