@@ -1,7 +1,16 @@
+Sequel::JDBC.load_driver('com.mysql.jdbc.Driver', :MySQL)
 Sequel.require 'adapters/shared/mysql'
 
 module Sequel
   module JDBC
+    Sequel.synchronize do
+      DATABASE_SETUP[:mysql] = proc do |db|
+        db.extend(Sequel::JDBC::MySQL::DatabaseMethods)
+        db.extend_datasets Sequel::MySQL::DatasetMethods
+        com.mysql.jdbc.Driver
+      end
+    end
+
     # Database and Dataset instance methods for MySQL specific
     # support via JDBC.
     module MySQL
@@ -26,6 +35,11 @@ module Sequel
           false
         end
 
+        # Raise a disconnect error if the SQL state of the cause of the exception indicates so.
+        def disconnect_error?(exception, opts)
+          exception.message =~ /\ACommunications link failure/ || super
+        end
+
         # Get the last inserted id using LAST_INSERT_ID().
         def last_insert_id(conn, opts=OPTS)
           if stmt = opts[:stmt]
