@@ -41,8 +41,9 @@ module Sequel
       include Sequel::MySQL::DatabaseMethods
       include Sequel::MySQL::PreparedStatements::DatabaseMethods
       
-      # Mysql::Error messages that indicate the current connection should be disconnected
-      MYSQL_DATABASE_DISCONNECT_ERRORS = /\A(Commands out of sync; you can't run this command now|Can't connect to local MySQL server through socket|MySQL server has gone away|Lost connection to MySQL server during query)/
+      # Regular expression used for getting accurate number of rows
+      # matched by an update statement.
+      AFFECTED_ROWS_RE = /Rows matched:\s+(\d+)\s+Changed:\s+\d+\s+Warnings:\s+\d+/.freeze
       
       set_adapter_scheme :mysql
 
@@ -58,34 +59,27 @@ module Sequel
       # it returns the strings as is.
       attr_reader :convert_invalid_date_time
       
-      def initialize(opts={})
-        super
-        @conversion_procs = MYSQL_TYPES.dup
-        self.convert_tinyint_to_bool = Sequel::MySQL.convert_tinyint_to_bool
-        self.convert_invalid_date_time = Sequel::MySQL.convert_invalid_date_time
-      end
-
       # Connect to the database.  In addition to the usual database options,
       # the following options have effect:
       #
-      # * :auto_is_null - Set to true to use MySQL default behavior of having
-      #   a filter for an autoincrement column equals NULL to return the last
-      #   inserted row.
-      # * :charset - Same as :encoding (:encoding takes precendence)
-      # * :compress - Set to false to not compress results from the server
-      # * :config_default_group - The default group to read from the in
-      #   the MySQL config file.
-      # * :config_local_infile - If provided, sets the Mysql::OPT_LOCAL_INFILE
-      #   option on the connection with the given value.
-      # * :connect_timeout - Set the timeout in seconds before a connection
-      #   attempt is abandoned.
-      # * :encoding - Set all the related character sets for this
-      #   connection (connection, client, database, server, and results).
-      # * :read_timeout - Set the timeout in seconds for reading back results
-      #   to a query.
-      # * :socket - Use a unix socket file instead of connecting via TCP/IP.
-      # * :timeout - Set the timeout in seconds before the server will
-      #   disconnect this connection (a.k.a @@wait_timeout).
+      # :auto_is_null :: Set to true to use MySQL default behavior of having
+      #                  a filter for an autoincrement column equals NULL to return the last
+      #                  inserted row.
+      # :charset :: Same as :encoding (:encoding takes precendence)
+      # :compress :: Set to false to not compress results from the server
+      # :config_default_group :: The default group to read from the in
+      #                          the MySQL config file.
+      # :config_local_infile :: If provided, sets the Mysql::OPT_LOCAL_INFILE
+      #                         option on the connection with the given value.
+      # :connect_timeout :: Set the timeout in seconds before a connection
+      #                     attempt is abandoned.
+      # :encoding :: Set all the related character sets for this
+      #              connection (connection, client, database, server, and results).
+      # :read_timeout :: Set the timeout in seconds for reading back results
+      #                  to a query.
+      # :socket :: Use a unix socket file instead of connecting via TCP/IP.
+      # :timeout :: Set the timeout in seconds before the server will
+      #             disconnect this connection (a.k.a @@wait_timeout).
       def connect(server)
         opts = server_opts(server)
         conn = Mysql.init
@@ -114,19 +108,13 @@ module Sequel
           Mysql::CLIENT_MULTI_STATEMENTS +
           (opts[:compress] == false ? 0 : Mysql::CLIENT_COMPRESS)
         )
-        sqls = []
+        sqls = mysql_connection_setting_sqls
+
         # Set encoding a slightly different way after connecting,
         # in case the READ_DEFAULT_GROUP overrode the provided encoding.
         # Doesn't work across implicit reconnects, but Sequel doesn't turn on
         # that feature.
-        sqls << "SET NAMES #{literal(encoding.to_s)}" if encoding
-
-        # Increase timeout so mysql server doesn't disconnect us
-        # Value used by default is maximum allowed value on Windows.
-        sqls << "SET @@wait_timeout = #{opts[:timeout] || 2147483}"
-
-        # By default, MySQL 'where id is null' selects the last inserted id
-        sqls << "SET SQL_AUTO_IS_NULL=0" unless opts[:auto_is_null]
+        sqls.unshift("SET NAMES #{literal(encoding.to_s)}") if encoding
 
         sqls.each{|sql| log_yield(sql){conn.query(sql)}}
 
@@ -134,6 +122,13 @@ module Sequel
         conn
       end
       
+      # Closes given database connection.
+      def disconnect_connection(c)
+        c.close
+      rescue Mysql::Error
+        nil
+      end
+      
       # Modify the type translators for the date, time, and timestamp types
       # depending on the value given.
       def convert_invalid_date_time=(v)
@@ -155,6 +150,16 @@ module Sequel
         @convert_tinyint_to_bool = v
       end
 
+      # Return the number of matched rows when executing a delete/update statement.
+      def execute_dui(sql, opts=OPTS)
+        execute(sql, opts){|c| return affected_rows(c)}
+      end
+
+      # Return the last inserted id when executing an insert statement.
+      def execute_insert(sql, opts=OPTS)
+        execute(sql, opts){|c| return c.insert_id}
+      end
+
       # Return the version of the MySQL server two which we are connecting.
       def server_version(server=nil)
         @server_version ||= (synchronize(server){|conn| conn.server_version if conn.respond_to?(:server_version)} || super)
@@ -167,7 +172,7 @@ module Sequel
       # yield the connection if a block is given.
       def _execute(conn, sql, opts)
         begin
-          r = log_yield(sql){conn.query(sql)}
+          r = log_yield((log_sql = opts[:log_sql]) ? sql + log_sql : sql){conn.query(sql)}
           if opts[:type] == :select
             yield r if r
           elsif block_given?
@@ -209,6 +214,24 @@ module Sequel
         end
       end
       
+      def adapter_initialize
+        @conversion_procs = MYSQL_TYPES.dup
+        self.convert_tinyint_to_bool = Sequel::MySQL.convert_tinyint_to_bool
+        self.convert_invalid_date_time = Sequel::MySQL.convert_invalid_date_time
+      end
+
+      # Try to get an accurate number of rows matched using the query
+      # info.  Fall back to affected_rows if there was no match, but
+      # that may be inaccurate.
+      def affected_rows(conn)
+        s = conn.info
+        if s && s =~ AFFECTED_ROWS_RE
+          $1.to_i
+        else
+          conn.affected_rows
+        end
+      end
+
       # MySQL connections use the query method to execute SQL without a result
       def connection_execute_method
         :query
@@ -237,6 +260,10 @@ module Sequel
         [Mysql::Error]
       end
 
+      def database_exception_sqlstate(exception, opts)
+        exception.sqlstate
+      end
+
       # Raise a disconnect error if the exception message matches the list
       # of recognized exceptions.
       def disconnect_error?(e, opts)
@@ -249,16 +276,9 @@ module Sequel
         @opts[:database]
       end
       
-      # Closes given database connection.
-      def disconnect_connection(c)
-        c.close
-      rescue Mysql::Error
-        nil
-      end
-      
       # Convert tinyint(1) type to boolean if convert_tinyint_to_bool is true
       def schema_column_type(db_type)
-        convert_tinyint_to_bool && db_type == 'tinyint(1)' ? :boolean : super
+        convert_tinyint_to_bool && db_type =~ /\Atinyint\(1\)/ ? :boolean : super
       end
     end
     
@@ -269,19 +289,10 @@ module Sequel
 
       Database::DatasetClass = self
 
-      # Regular expression used for getting accurate number of rows
-      # matched by an update statement.
-      AFFECTED_ROWS_RE = /Rows matched:\s+(\d+)\s+Changed:\s+\d+\s+Warnings:\s+\d+/.freeze
-      
-      # Delete rows matching this dataset
-      def delete
-        execute_dui(delete_sql){|c| return c.affected_rows}
-      end
-      
       # Yield all rows matching this dataset.  If the dataset is set to
       # split multiple statements, yield arrays of hashes one per statement
       # instead of yielding results for all statements as hashes.
-      def fetch_rows(sql, &block)
+      def fetch_rows(sql)
         execute(sql) do |r|
           i = -1
           cps = db.conversion_procs
@@ -289,7 +300,7 @@ module Sequel
             # Pretend tinyint is another integer type if its length is not 1, to
             # avoid casting to boolean if Sequel::MySQL.convert_tinyint_to_bool
             # is set.
-            type_proc = f.type == 1 && f.length != 1 ? cps[2] : cps[f.type]
+            type_proc = f.type == 1 && cast_tinyint_integer?(f) ? cps[2] : cps[f.type]
             [output_identifier(f.name), type_proc, i+=1]
           end
           @columns = cols.map{|c| c.first}
@@ -298,7 +309,7 @@ module Sequel
             yield_rows(r, cols){|h| s << h}
             yield s
           else
-            yield_rows(r, cols, &block)
+            yield_rows(r, cols){|h| yield h}
           end
         end
         self
@@ -310,22 +321,6 @@ module Sequel
         super
       end
       
-      # Insert a new value into this dataset
-      def insert(*values)
-        execute_dui(insert_sql(*values)){|c| return c.insert_id}
-      end
-      
-      # You can parse out the correct number of rows matched using the query info,
-      # even though affected_rows doesn't provide an accurate number.
-      def provides_accurate_rows_matched?
-        true
-      end
-
-      # Replace (update or insert) the matching row.
-      def replace(*args)
-        execute_dui(replace_sql(*args)){|c| return c.insert_id}
-      end
-      
       # Makes each yield arrays of rows, with each array containing the rows
       # for a given result set.  Does not work with graphing.  So you can submit
       # SQL with multiple statements and easily determine which statement
@@ -342,33 +337,18 @@ module Sequel
         ds
       end
       
-      # Update the matching rows.
-      def update(values={})
-        execute_dui(update_sql(values)){|c| return affected_rows(c)}
-      end
-      
       private
-      
-      # Try to get an accurate number of rows matched using the query
-      # info.  Fall back to affected_rows if there was no match, but
-      # that may be inaccurate.
-      def affected_rows(conn)
-        s = conn.info
-        if s && s =~ AFFECTED_ROWS_RE
-          $1.to_i
-        else
-          conn.affected_rows
-        end
-      end
 
-      # Set the :type option to :select if it hasn't been set.
-      def execute(sql, opts={}, &block)
-        super(sql, {:type=>:select}.merge(opts), &block)
+      # Whether a tinyint field should be casted as an integer.  By default,
+      # casts to integer if the field length is not 1.  Can be overwritten
+      # to make tinyint casting dataset dependent.
+      def cast_tinyint_integer?(field)
+        field.length != 1
       end
       
-      # Set the :type option to :dui if it hasn't been set.
-      def execute_dui(sql, opts={}, &block)
-        super(sql, {:type=>:dui}.merge(opts), &block)
+      # Set the :type option to :select if it hasn't been set.
+      def execute(sql, opts=OPTS, &block)
+        super(sql, {:type=>:select}.merge(opts), &block)
       end
       
       # Handle correct quoting of strings using ::MySQL.quote.
