@@ -6,15 +6,26 @@ module Sequel
     # prepared statements and stored procedures.
     module PreparedStatements
       module DatabaseMethods
+        disconnect_errors = <<-END.split("\n").map{|l| l.strip}
+        Commands out of sync; you can't run this command now
+        Can't connect to local MySQL server through socket
+        MySQL server has gone away
+        Lost connection to MySQL server during query
+        This connection is still waiting for a result, try again once you have the result
+        closed MySQL connection
+        END
+        # Error messages for mysql and mysql2 that indicate the current connection should be disconnected
+        MYSQL_DATABASE_DISCONNECT_ERRORS = /\A#{Regexp.union(disconnect_errors)}/o
+       
         # Support stored procedures on MySQL
-        def call_sproc(name, opts={}, &block)
+        def call_sproc(name, opts=OPTS, &block)
           args = opts[:args] || [] 
           execute("CALL #{name}#{args.empty? ? '()' : literal(args)}", opts.merge(:sproc=>false), &block)
         end
         
         # Executes the given SQL using an available connection, yielding the
         # connection if the block is given.
-        def execute(sql, opts={}, &block)
+        def execute(sql, opts=OPTS, &block)
           if opts[:sproc]
             call_sproc(sql, opts, &block)
           elsif sql.is_a?(Symbol)
@@ -33,6 +44,22 @@ module Sequel
           conn.prepared_statements = {}
         end
 
+        # Stupid MySQL doesn't use SQLState error codes correctly, mapping
+        # all constraint violations to 23000 even though it recognizes
+        # different types.
+        def database_specific_error_class(exception, opts)
+          case exception.errno
+          when 1048
+            NotNullConstraintViolation
+          when 1062
+            UniqueConstraintViolation
+          when 1451, 1452
+            ForeignKeyConstraintViolation
+          else
+            super
+          end
+        end
+
         # Executes a prepared statement on an available connection.  If the
         # prepared statement already exists for the connection and has the same
         # SQL, reuse it, otherwise, prepare the new statement.  Because of the
@@ -45,8 +72,8 @@ module Sequel
           sql = ps.prepared_sql
           synchronize(opts[:server]) do |conn|
             unless conn.prepared_statements[ps_name] == sql
-              conn.prepared_statements[ps_name] = sql
               _execute(conn, "PREPARE #{ps_name} FROM #{literal(sql)}", opts)
+              conn.prepared_statements[ps_name] = sql
             end
             i = 0
             _execute(conn, "SET " + args.map {|arg| "@sequel_arg_#{i+=1} = #{literal(arg)}"}.join(", "), opts) unless args.empty?
@@ -54,8 +81,8 @@ module Sequel
             _execute(conn, "EXECUTE #{ps_name}#{" USING #{(1..i).map{|j| "@sequel_arg_#{j}"}.join(', ')}" unless i == 0}", opts, &block)
           end
         end
-        
       end
+
       module DatasetMethods
         include Sequel::Dataset::StoredProcedures
        
@@ -87,17 +114,17 @@ module Sequel
           
           # Execute the prepared statement with the bind arguments instead of
           # the given SQL.
-          def execute(sql, opts={}, &block)
+          def execute(sql, opts=OPTS, &block)
             super(prepared_statement_name, {:arguments=>bind_arguments}.merge(opts), &block)
           end
           
           # Same as execute, explicit due to intricacies of alias and super.
-          def execute_dui(sql, opts={}, &block)
+          def execute_dui(sql, opts=OPTS, &block)
             super(prepared_statement_name, {:arguments=>bind_arguments}.merge(opts), &block)
           end
           
           # Same as execute, explicit due to intricacies of alias and super.
-          def execute_insert(sql, opts={}, &block)
+          def execute_insert(sql, opts=OPTS, &block)
             super(prepared_statement_name, {:arguments=>bind_arguments}.merge(opts), &block)
           end
         end
@@ -109,12 +136,12 @@ module Sequel
           private
           
           # Execute the database stored procedure with the stored arguments.
-          def execute(sql, opts={}, &block)
+          def execute(sql, opts=OPTS, &block)
             super(@sproc_name, {:args=>@sproc_args, :sproc=>true}.merge(opts), &block)
           end
           
           # Same as execute, explicit due to intricacies of alias and super.
-          def execute_dui(sql, opts={}, &block)
+          def execute_dui(sql, opts=OPTS, &block)
             super(@sproc_name, {:args=>@sproc_args, :sproc=>true}.merge(opts), &block)
           end
         end
