@@ -161,59 +161,60 @@ module Sequel
       end
 
       # Creates the function in the database.  Arguments:
-      # * name : name of the function to create
-      # * definition : string definition of the function, or object file for a dynamically loaded C function.
-      # * opts : options hash:
-      #   * :args : function arguments, can be either a symbol or string specifying a type or an array of 1-3 elements:
-      #     * element 1 : argument data type
-      #     * element 2 : argument name
-      #     * element 3 : argument mode (e.g. in, out, inout)
-      #   * :behavior : Should be IMMUTABLE, STABLE, or VOLATILE.  PostgreSQL assumes VOLATILE by default.
-      #   * :cost : The estimated cost of the function, used by the query planner.
-      #   * :language : The language the function uses.  SQL is the default.
-      #   * :link_symbol : For a dynamically loaded see function, the function's link symbol if different from the definition argument.
-      #   * :returns : The data type returned by the function.  If you are using OUT or INOUT argument modes, this is ignored.
-      #     Otherwise, if this is not specified, void is used by default to specify the function is not supposed to return a value.
-      #   * :rows : The estimated number of rows the function will return.  Only use if the function returns SETOF something.
-      #   * :security_definer : Makes the privileges of the function the same as the privileges of the user who defined the function instead of
-      #     the privileges of the user who runs the function.  There are security implications when doing this, see the PostgreSQL documentation.
-      #   * :set : Configuration variables to set while the function is being run, can be a hash or an array of two pairs.  search_path is
-      #     often used here if :security_definer is used.
-      #   * :strict : Makes the function return NULL when any argument is NULL.
+      # name :: name of the function to create
+      # definition :: string definition of the function, or object file for a dynamically loaded C function.
+      # opts :: options hash:
+      #         :args :: function arguments, can be either a symbol or string specifying a type or an array of 1-3 elements:
+      #                  1 :: argument data type
+      #                  2 :: argument name
+      #                  3 :: argument mode (e.g. in, out, inout)
+      #         :behavior :: Should be IMMUTABLE, STABLE, or VOLATILE.  PostgreSQL assumes VOLATILE by default.
+      #         :cost :: The estimated cost of the function, used by the query planner.
+      #         :language :: The language the function uses.  SQL is the default.
+      #         :link_symbol :: For a dynamically loaded see function, the function's link symbol if different from the definition argument.
+      #         :returns :: The data type returned by the function.  If you are using OUT or INOUT argument modes, this is ignored.
+      #                     Otherwise, if this is not specified, void is used by default to specify the function is not supposed to return a value.
+      #         :rows :: The estimated number of rows the function will return.  Only use if the function returns SETOF something.
+      #         :security_definer :: Makes the privileges of the function the same as the privileges of the user who defined the function instead of
+      #                              the privileges of the user who runs the function.  There are security implications when doing this, see the PostgreSQL documentation.
+      #         :set :: Configuration variables to set while the function is being run, can be a hash or an array of two pairs.  search_path is
+      #                 often used here if :security_definer is used.
+      #         :strict :: Makes the function return NULL when any argument is NULL.
       def create_function(name, definition, opts=OPTS)
         self << create_function_sql(name, definition, opts)
       end
 
       # Create the procedural language in the database. Arguments:
-      # * name : Name of the procedural language (e.g. plpgsql)
-      # * opts : options hash:
-      #   * :handler : The name of a previously registered function used as a call handler for this language.
-      #   * :replace : Replace the installed language if it already exists (on PostgreSQL 9.0+).
-      #   * :trusted : Marks the language being created as trusted, allowing unprivileged users to create functions using this language.
-      #   * :validator : The name of previously registered function used as a validator of functions defined in this language.
+      # name :: Name of the procedural language (e.g. plpgsql)
+      # opts :: options hash:
+      #         :handler :: The name of a previously registered function used as a call handler for this language.
+      #         :replace :: Replace the installed language if it already exists (on PostgreSQL 9.0+).
+      #         :trusted :: Marks the language being created as trusted, allowing unprivileged users to create functions using this language.
+      #         :validator :: The name of previously registered function used as a validator of functions defined in this language.
       def create_language(name, opts=OPTS)
         self << create_language_sql(name, opts)
       end
 
       # Create a schema in the database. Arguments:
-      # * name : Name of the schema (e.g. admin)
-      # * opts : options hash:
-      #   * :if_not_exists : Don't raise an error if the schema already exists (PostgreSQL 9.3+)
-      #   * :owner : The owner to set for the schema (defaults to current user if not specified)
+      # name :: Name of the schema (e.g. admin)
+      # opts :: options hash:
+      #         :if_not_exists :: Don't raise an error if the schema already exists (PostgreSQL 9.3+)
+      #         :owner :: The owner to set for the schema (defaults to current user if not specified)
       def create_schema(name, opts=OPTS)
         self << create_schema_sql(name, opts)
       end
 
       # Create a trigger in the database.  Arguments:
-      # * table : the table on which this trigger operates
-      # * name : the name of this trigger
-      # * function : the function to call for this trigger, which should return type trigger.
-      # * opts : options hash:
-      #   * :after : Calls the trigger after execution instead of before.
-      #   * :args : An argument or array of arguments to pass to the function.
-      #   * :each_row : Calls the trigger for each row instead of for each statement.
-      #   * :events : Can be :insert, :update, :delete, or an array of any of those. Calls the trigger whenever that type of statement is used.  By default,
-      #     the trigger is called for insert, update, or delete.
+      # table :: the table on which this trigger operates
+      # name :: the name of this trigger
+      # function :: the function to call for this trigger, which should return type trigger.
+      # opts :: options hash:
+      #         :after :: Calls the trigger after execution instead of before.
+      #         :args :: An argument or array of arguments to pass to the function.
+      #         :each_row :: Calls the trigger for each row instead of for each statement.
+      #         :events :: Can be :insert, :update, :delete, or an array of any of those. Calls the trigger whenever that type of statement is used.  By default,
+      #                    the trigger is called for insert, update, or delete.
+      #         :when :: A filter to use for the trigger
       def create_trigger(table, name, function, opts=OPTS)
         self << create_trigger_sql(table, name, function, opts)
       end
@@ -234,39 +235,39 @@ module Sequel
       end
 
       # Drops the function from the database. Arguments:
-      # * name : name of the function to drop
-      # * opts : options hash:
-      #   * :args : The arguments for the function.  See create_function_sql.
-      #   * :cascade : Drop other objects depending on this function.
-      #   * :if_exists : Don't raise an error if the function doesn't exist.
+      # name :: name of the function to drop
+      # opts :: options hash:
+      #         :args :: The arguments for the function.  See create_function_sql.
+      #         :cascade :: Drop other objects depending on this function.
+      #         :if_exists :: Don't raise an error if the function doesn't exist.
       def drop_function(name, opts=OPTS)
         self << drop_function_sql(name, opts)
       end
 
       # Drops a procedural language from the database.  Arguments:
-      # * name : name of the procedural language to drop
-      # * opts : options hash:
-      #   * :cascade : Drop other objects depending on this function.
-      #   * :if_exists : Don't raise an error if the function doesn't exist.
+      # name :: name of the procedural language to drop
+      # opts :: options hash:
+      #         :cascade :: Drop other objects depending on this function.
+      #         :if_exists :: Don't raise an error if the function doesn't exist.
       def drop_language(name, opts=OPTS)
         self << drop_language_sql(name, opts)
       end
 
       # Drops a schema from the database.  Arguments:
-      # * name : name of the schema to drop
-      # * opts : options hash:
-      #   * :cascade : Drop all objects in this schema.
-      #   * :if_exists : Don't raise an error if the schema doesn't exist.
+      # name :: name of the schema to drop
+      # opts :: options hash:
+      #         :cascade :: Drop all objects in this schema.
+      #         :if_exists :: Don't raise an error if the schema doesn't exist.
       def drop_schema(name, opts=OPTS)
         self << drop_schema_sql(name, opts)
       end
 
       # Drops a trigger from the database.  Arguments:
-      # * table : table from which to drop the trigger
-      # * name : name of the trigger to drop
-      # * opts : options hash:
-      #   * :cascade : Drop other objects depending on this function.
-      #   * :if_exists : Don't raise an error if the function doesn't exist.
+      # table :: table from which to drop the trigger
+      # name :: name of the trigger to drop
+      # opts :: options hash:
+      #         :cascade :: Drop other objects depending on this function.
+      #         :if_exists :: Don't raise an error if the function doesn't exist.
       def drop_trigger(table, name, opts=OPTS)
         self << drop_trigger_sql(table, name, opts)
       end
@@ -482,6 +483,11 @@ module Sequel
         true
       end
 
+      # PostgreSQL 9.0+ supports trigger conditions.
+      def supports_trigger_conditions?
+        server_version >= 90000
+      end
+
       # PostgreSQL supports prepared transactions (two-phase commit) if
       # max_prepared_transactions is greater than 0.
       def supports_prepared_transactions?
@@ -524,6 +530,17 @@ module Sequel
         @supported_types.fetch(type){@supported_types[type] = (from(:pg_type).filter(:typtype=>'b', :typname=>type.to_s).count > 0)}
       end
 
+      # Creates a dataset that uses the VALUES clause:
+      #
+      #   DB.values([[1, 2], [3, 4]])
+      #   VALUES ((1, 2), (3, 4))
+      #
+      #   DB.values([[1, 2], [3, 4]]).order(:column2).limit(1, 1)
+      #   VALUES ((1, 2), (3, 4)) ORDER BY column2 LIMIT 1 OFFSET 1
+      def values(v)
+        @default_dataset.clone(:values=>v)
+      end
+
       # Array of symbols specifying view names in the current database.
       #
       # Options:
@@ -697,9 +714,12 @@ module Sequel
       end
 
       EXCLUSION_CONSTRAINT_SQL_STATE = '23P01'.freeze
+      DEADLOCK_SQL_STATE = '40P01'.freeze
       def database_specific_error_class_from_sqlstate(sqlstate)
         if sqlstate == EXCLUSION_CONSTRAINT_SQL_STATE
           ExclusionConstraintViolation
+        elsif sqlstate == DEADLOCK_SQL_STATE
+          SerializationFailure
         else
           super
         end
@@ -789,27 +809,40 @@ module Sequel
 
       # DDL statement for creating a table with the given name, columns, and options
       def create_table_prefix_sql(name, options)
-        if on_commit = options[:on_commit]
-          raise(Error, "can't provide :on_commit without :temp to create_table") unless options[:temp]
-          raise(Error, "unsupported on_commit option: #{on_commit.inspect}") unless ON_COMMIT.has_key? on_commit
+        prefix_sql = if options[:temp]
+          raise(Error, "can't provide both :temp and :unlogged to create_table") if options[:unlogged]
+          raise(Error, "can't provide both :temp and :foreign to create_table") if options[:foreign]
+          temporary_table_sql
+        elsif options[:foreign]
+          raise(Error, "can't provide both :foreign and :unlogged to create_table") if options[:unlogged]
+          'FOREIGN '
+        elsif options[:unlogged]
+          UNLOGGED
         end
-        temp_or_unlogged_sql = if options[:temp]
-         raise(Error, "can't provide both :temp and :unlogged to create_table") if options[:unlogged]
-         temporary_table_sql
-       elsif options[:unlogged]
-         UNLOGGED
-       end
-        "CREATE #{temp_or_unlogged_sql}TABLE#{' IF NOT EXISTS' if options[:if_not_exists]} #{options[:temp] ? quote_identifier(name) : quote_schema_table(name)}"
+
+        "CREATE #{prefix_sql}TABLE#{' IF NOT EXISTS' if options[:if_not_exists]} #{options[:temp] ? quote_identifier(name) : quote_schema_table(name)}"
       end
 
       def create_table_sql(name, generator, options)
         sql = super
+
         if inherits = options[:inherits]
           sql << " INHERITS (#{Array(inherits).map{|t| quote_schema_table(t)}.join(', ')})"
         end
+
         if on_commit = options[:on_commit]
+          raise(Error, "can't provide :on_commit without :temp to create_table") unless options[:temp]
+          raise(Error, "unsupported on_commit option: #{on_commit.inspect}") unless ON_COMMIT.has_key?(on_commit)
           sql << " ON COMMIT #{ON_COMMIT[on_commit]}"
         end
+
+        if server = options[:foreign]
+          sql << " SERVER #{quote_identifier(server)}"
+          if foreign_opts = options[:options]
+            sql << " OPTIONS (#{foreign_opts.map{|k, v| "#{k} #{literal(v.to_s)}"}.join(', ')})"
+          end
+        end
+
         sql
       end
 
@@ -830,7 +863,11 @@ module Sequel
       def create_trigger_sql(table, name, function, opts=OPTS)
         events = opts[:events] ? Array(opts[:events]) : [:insert, :update, :delete]
         whence = opts[:after] ? 'AFTER' : 'BEFORE'
-        "CREATE TRIGGER #{name} #{whence} #{events.map{|e| e.to_s.upcase}.join(' OR ')} ON #{quote_schema_table(table)}#{' FOR EACH ROW' if opts[:each_row]} EXECUTE PROCEDURE #{function}(#{Array(opts[:args]).map{|a| literal(a)}.join(', ')})"
+        if filter = opts[:when]
+          raise Error, "Trigger conditions are not supported for this database" unless supports_trigger_conditions?
+          filter = " WHEN #{filter_expr(filter)}"
+        end
+        "CREATE TRIGGER #{name} #{whence} #{events.map{|e| e.to_s.upcase}.join(' OR ')} ON #{quote_schema_table(table)}#{' FOR EACH ROW' if opts[:each_row]}#{filter} EXECUTE PROCEDURE #{function}(#{Array(opts[:args]).map{|a| literal(a)}.join(', ')})"
       end
 
       # DDL fragment for initial part of CREATE VIEW statement
@@ -1147,12 +1184,13 @@ module Sequel
       CRLF = "\r\n".freeze
       BLOB_RE = /[\000-\037\047\134\177-\377]/n.freeze
       WINDOW = " WINDOW ".freeze
+      SELECT_VALUES = "VALUES ".freeze
       EMPTY_STRING = ''.freeze
       LOCK_MODES = ['ACCESS SHARE', 'ROW SHARE', 'ROW EXCLUSIVE', 'SHARE UPDATE EXCLUSIVE', 'SHARE', 'SHARE ROW EXCLUSIVE', 'EXCLUSIVE', 'ACCESS EXCLUSIVE'].each{|s| s.freeze}
 
       Dataset.def_sql_method(self, :delete, [['if server_version >= 90100', %w'with delete from using where returning'], ['else', %w'delete from using where returning']])
       Dataset.def_sql_method(self, :insert, [['if server_version >= 90100', %w'with insert into columns values returning'], ['else', %w'insert into columns values returning']])
-      Dataset.def_sql_method(self, :select, [['if server_version >= 80400', %w'with select distinct columns from join where group having window compounds order limit lock'], ['else', %w'select distinct columns from join where group having compounds order limit lock']])
+      Dataset.def_sql_method(self, :select, [['if opts[:values]', %w'values order limit'], ['elsif server_version >= 80400', %w'with select distinct columns from join where group having window compounds order limit lock'], ['else', %w'select distinct columns from join where group having compounds order limit lock']])
       Dataset.def_sql_method(self, :update, [['if server_version >= 90100', %w'with update table set from where returning'], ['else', %w'update table set from where returning']])
 
       # Shared methods for prepared statements when used with PostgreSQL databases.
@@ -1285,7 +1323,15 @@ module Sequel
       # Insert a record returning the record inserted.  Always returns nil without
       # inserting a query if disable_insert_returning is used.
       def insert_select(*values)
-        returning.insert(*values){|r| return r} unless @opts[:disable_insert_returning]
+        return unless supports_insert_select?
+        with_sql_first(insert_select_sql(*values))
+      end
+
+      # The SQL to use for an insert_select, adds a RETURNING clause to the insert
+      # unless the RETURNING clause is already present.
+      def insert_select_sql(*values)
+        ds = opts[:returning] ? self : returning
+        ds.insert_sql(*values)
       end
 
       # Locks all tables in the dataset's FROM clause (but not in JOINs) with
@@ -1367,7 +1413,7 @@ module Sequel
       #
       # Options:
       # :cascade :: whether to use the CASCADE option, useful when truncating
-      #   tables with Foreign Keys.
+      #             tables with foreign keys.
       # :only :: truncate using ONLY, so child tables are unaffected
       # :restart :: use RESTART IDENTITY to restart any related sequences
       #
@@ -1510,6 +1556,12 @@ module Sequel
         @opts[:lock] == :share ? (sql << FOR_SHARE) : super
       end
 
+      # Support VALUES clause instead of the SELECT clause to return rows.
+      def select_values_sql(sql)
+        sql << SELECT_VALUES
+        expression_list_append(sql, opts[:values])
+      end
+
       # SQL fragment for named window specifications
       def select_window_sql(sql)
         if ws = @opts[:window]
