@@ -6,6 +6,7 @@
 #
 #   Sequel.extension :migration
 
+#
 module Sequel
   # Sequel's older migration class, available for backward compatibility.
   # Uses subclasses with up and down instance methods for each migration:
@@ -206,8 +207,8 @@ module Sequel
       @actions << [:drop_table, args.first]
     end
 
-    def create_view(*args)
-      @actions << [:drop_view, args.first]
+    def create_view(name, _, options={})
+      @actions << [:drop_view, name, options]
     end
 
     def rename_column(table, name, new_name)
@@ -373,7 +374,7 @@ module Sequel
       migrator_class(directory).new(db, directory, opts).is_current?
     end
 
-    # Migrates the supplied database using the migration files in the the specified directory. Options:
+    # Migrates the supplied database using the migration files in the specified directory. Options:
     # :allow_missing_migration_files :: Don't raise an error if there are missing migration files.
     # :column :: The column in the :table argument storing the migration version (default: :version).
     # :current :: The current version of the database.  If not given, it is retrieved from the database
@@ -404,7 +405,6 @@ module Sequel
         self
       end
     end
-    private_class_method :migrator_class
     
     # The column to use to hold the migration version number for integer migrations or
     # filename for timestamp migrations (defaults to :version for integer migrations and
