@@ -85,6 +85,14 @@ module Sequel
           initial_values.has_key?(column)
         end
 
+        # Freeze internal data structures
+        def freeze
+          initial_values.freeze
+          missing_initial_values.freeze
+          @previous_changes.freeze if @previous_changes
+          super
+        end
+
         # The initial value of the given column.  If the column value has
         # not changed, this will be the same as the current value of the
         # column.
@@ -101,14 +109,6 @@ module Sequel
           @initial_values ||= {}
         end
 
-        # Freeze internal data structures
-        def freeze
-          initial_values.freeze
-          missing_initial_values.freeze
-          @previous_changes.freeze if @previous_changes
-          super
-        end
-
         # Reset the column to its initial value.  If the column was not set
         # initial, removes it from the values.
         #
@@ -198,6 +198,15 @@ module Sequel
           end
         end
 
+        # Duplicate internal data structures
+        def initialize_copy(other)
+          super
+          @initial_values = other.initial_values.dup
+          @missing_initial_values = other.send(:missing_initial_values).dup
+          @previous_changes = other.previous_changes.dup if other.previous_changes
+          self
+        end
+
         # Reset the initial values when initializing.
         def initialize_set(h)
           super
