@@ -54,6 +54,15 @@ module Sequel
 
         private
 
+        # Duplicate the original deserialized values when duplicating instance.
+        def initialize_copy(other)
+          super
+          if o = other.instance_variable_get(:@original_deserialized_values)
+            @original_deserialized_values = o.dup
+          end
+          self
+        end
+
         # For new objects, serialize any existing deserialized values so that changes can
         # be detected.
         def initialize_set(values)
