@@ -19,7 +19,7 @@ module Sequel
     # Options:
     # :allow_blank :: Whether to skip the validation if the value is blank.  You should
     #                 make sure all objects respond to blank if you use this option, which you can do by:
-    #     Sequel.extension :blank
+    #                     Sequel.extension :blank
     # :allow_missing :: Whether to skip the validation if the attribute isn't a key in the
     #                   values hash.  This is different from allow_nil, because Sequel only sends the attributes
     #                   in the values when doing an insert or update.  If the attribute is not present, Sequel
@@ -44,8 +44,7 @@ module Sequel
     #
     #   Sequel::Plugins::ValidationHelpers::DEFAULT_OPTIONS.merge!(
     #     :exact_length=>{:message=>lambda{|exact| I18n.t("errors.exact_length", :exact => exact)}},
-    #     :integer=>{:message=>lambda{I18n.t("errors.integer")}},
-    #     ...
+    #     :integer=>{:message=>lambda{I18n.t("errors.integer")}}
     #   )
     #
     # and then use something like this in your yaml translation file:
@@ -202,6 +201,8 @@ module Sequel
         # since it can deal with a grouping of multiple attributes.
         #
         # Possible Options:
+        # :dataset :: The base dataset to use for the unique query, defaults to the
+        #             model's dataset.
         # :message :: The message to use (default: 'is already taken')
         # :only_if_modified :: Only check the uniqueness if the object is new or
         #                      one of the columns has been modified.
@@ -213,7 +214,7 @@ module Sequel
         # If you want to to a case insensitive uniqueness validation on a database that
         # is case sensitive by default, you can use:
         #
-        #   :where=>(proc do |ds, obj, cols|
+        #   validates_unique :column, :where=>(proc do |ds, obj, cols|
         #     ds.where(cols.map do |c|
         #       v = obj.send(c)
         #       v = v.downcase if v
@@ -231,12 +232,13 @@ module Sequel
             arr = Array(a)
             next if arr.any?{|x| errors.on(x)}
             next if opts[:only_if_modified] && !new? && !arr.any?{|x| changed_columns.include?(x)}
+            ds = opts[:dataset] || model.dataset
             ds = if where
-              where.call(model.dataset, self, arr)
+              where.call(ds, self, arr)
             else
               vals = arr.map{|x| send(x)}
               next if vals.any?{|v| v.nil?}
-              model.where(arr.zip(vals))
+              ds.where(arr.zip(vals))
             end
             ds = yield(ds) if block_given?
             ds = ds.exclude(pk_hash) unless new?
