@@ -909,7 +909,7 @@ module Sequel
       # Return a +StringExpression+ representing the concatenation of the receiver
       # with the given argument.
       #
-      #   :x.sql_string + :y => # "x" || "y"
+      #   :x.sql_string + :y # => "x" || "y"
       def +(ce)
         StringExpression.new(:'||', self, ce)
       end
@@ -984,14 +984,14 @@ module Sequel
       # and converts it to a +BooleanExpression+.  The operator and args
       # used depends on the case of the right (2nd) argument:
       #
-      # * 0..10 - left >= 0 AND left <= 10
-      # * [1,2] - left IN (1,2)
-      # * nil - left IS NULL
-      # * true - left IS TRUE 
-      # * false - left IS FALSE 
-      # * /as/ - left ~ 'as'
-      # * :blah - left = blah
-      # * 'blah' - left = 'blah'
+      # 0..10 :: left >= 0 AND left <= 10
+      # [1,2] :: left IN (1,2)
+      # nil :: left IS NULL
+      # true :: left IS TRUE 
+      # false :: left IS FALSE 
+      # /as/ :: left ~ 'as'
+      # :blah :: left = blah
+      # 'blah' :: left = 'blah'
       #
       # If multiple arguments are given, they are joined with the op given (AND
       # by default, OR possible).  If negate is set to true,
@@ -1022,7 +1022,7 @@ module Sequel
         when Regexp
           StringExpression.like(l, r)
         when DelayedEvaluation
-          Sequel.delay{from_value_pair(l, r.callable.call)}
+          Sequel.delay{|ds| from_value_pair(l, r.call(ds))}
         when Dataset::PlaceholderLiteralizer::Argument
           r.transform{|v| from_value_pair(l, v)}
         else
@@ -1220,7 +1220,18 @@ module Sequel
         @callable = callable
       end
 
-      to_s_method :delayed_evaluation_sql, '@callable'
+      # Call the underlying callable and return the result.  If the
+      # underlying callable only accepts a single argument, call it
+      # with the given dataset.
+      def call(ds)
+        if @callable.respond_to?(:arity) && @callable.arity == 1
+          @callable.call(ds)
+        else
+          @callable.call
+        end
+      end
+
+      to_s_method :delayed_evaluation_sql
     end
 
     # Represents an SQL function call.
@@ -1339,16 +1350,6 @@ module Sequel
       end
     end
 
-    # REMOVE411
-    class EmulatedFunction < Function
-      def self.new(name, *args)
-        Deprecation.deprecate("Sequel::SQL::EmulatedFunction", "Please use Sequel::SQL::Function.new!(name, args, :emulate=>true) to create an emulated SQL function")
-        Function.new!(name, args, :emulate=>true)
-      end
-
-      to_s_method :emulated_function_sql
-    end
-    
     class GenericExpression
       include AliasMethods
       include BooleanMethods
@@ -1394,18 +1395,9 @@ module Sequel
       attr_reader :table_expr
 
       # Create an object with the given join_type and table expression.
-      def initialize(join_type, table, table_alias = nil)
+      def initialize(join_type, table_expr)
         @join_type = join_type
-
-        @table_expr = if table.is_a?(AliasedExpression)
-          table
-        # REMOVE411
-        elsif table_alias
-          Deprecation.deprecate("The table_alias argument to Sequel::SQL::JoinClause#initialize", "Please use a Sequel::SQL::AliasedExpression as the table argument instead.")
-          AliasedExpression.new(table, table_alias)
-        else
-          table
-        end
+        @table_expr = table_expr
       end
 
       # The table/set related to the JOIN, without any alias.
@@ -1827,28 +1819,6 @@ module Sequel
       to_s_method :window_sql, '@opts'
     end
 
-    # REMOVE411
-    class WindowFunction < GenericExpression
-      # The function to use, should be an <tt>SQL::Function</tt>.
-      attr_reader :function
-
-      # The window to use, should be an <tt>SQL::Window</tt>.
-      attr_reader :window
-
-      def self.new(function, window)
-        Deprecation.deprecate("Sequel::SQL::WindowFunction", "Please use Sequel::SQL::Function.new(name, *args).over(...) to create an SQL window function")
-        function.over(window)
-      end
-
-      # Set the function and window.
-      def initialize(function, window)
-        Deprecation.deprecate("Sequel::SQL::WindowFunction", "Please use Sequel::SQL::Function.new(name, *args).over(...) to create an SQL window function")
-        @function, @window = function, window
-      end
-
-      to_s_method :window_function_sql, '@function, @window'
-    end
-
     # A +Wrapper+ is a simple way to wrap an existing object so that it supports
     # the Sequel DSL.
     class Wrapper < GenericExpression
