@@ -1,9 +1,18 @@
 require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper')
 CONNECTION_POOL_DEFAULTS = {:pool_timeout=>5, :pool_sleep_time=>0.001, :max_connections=>4}
 
+mock_db = lambda do |*a, &b|
+  db = Sequel.mock
+  (class << db; self end).send(:define_method, :connect){|c| b.arity == 1 ? b.call(c) : b.call} if b
+  if b2 = a.shift
+    (class << db; self end).send(:define_method, :disconnect_connection){|c| b2.arity == 1 ? b2.call(c) : b2.call}
+  end
+  db
+end
+
 describe "An empty ConnectionPool" do
   before do
-    @cpool = Sequel::ConnectionPool.get_pool(CONNECTION_POOL_DEFAULTS){}
+    @cpool = Sequel::ConnectionPool.get_pool(mock_db.call, CONNECTION_POOL_DEFAULTS)
   end
 
   specify "should have no available connections" do
@@ -21,24 +30,25 @@ end
 
 describe "ConnectionPool options" do
   specify "should support string option values" do
-    cpool = Sequel::ConnectionPool.get_pool({:max_connections=>'5', :pool_timeout=>'3', :pool_sleep_time=>'0.01'}){}
+    cpool = Sequel::ConnectionPool.get_pool(mock_db.call, {:max_connections=>'5', :pool_timeout=>'3', :pool_sleep_time=>'0.01'})
     cpool.max_size.should == 5
     cpool.instance_variable_get(:@timeout).should == 3
     cpool.instance_variable_get(:@sleep_time).should == 0.01
   end
 
   specify "should raise an error unless size is positive" do
-    lambda{Sequel::ConnectionPool.get_pool(:max_connections=>0)}.should raise_error(Sequel::Error)
-    lambda{Sequel::ConnectionPool.get_pool(:max_connections=>-10)}.should raise_error(Sequel::Error)
-    lambda{Sequel::ConnectionPool.get_pool(:max_connections=>'-10')}.should raise_error(Sequel::Error)
-    lambda{Sequel::ConnectionPool.get_pool(:max_connections=>'0')}.should raise_error(Sequel::Error)
+    lambda{Sequel::ConnectionPool.get_pool(mock_db.call{1}, :max_connections=>0)}.should raise_error(Sequel::Error)
+    lambda{Sequel::ConnectionPool.get_pool(mock_db.call{1}, :max_connections=>-10)}.should raise_error(Sequel::Error)
+    lambda{Sequel::ConnectionPool.get_pool(mock_db.call{1}, :max_connections=>'-10')}.should raise_error(Sequel::Error)
+    lambda{Sequel::ConnectionPool.get_pool(mock_db.call{1}, :max_connections=>'0')}.should raise_error(Sequel::Error)
   end
 end
 
 describe "A connection pool handling connections" do
   before do
     @max_size = 2
-    @cpool = Sequel::ConnectionPool.get_pool(CONNECTION_POOL_DEFAULTS.merge(:disconnection_proc=>proc{|c| @max_size=3},  :max_connections=>@max_size)) {:got_connection}
+    msp = proc{@max_size=3}
+    @cpool = Sequel::ConnectionPool.get_pool(mock_db.call(proc{|c| msp.call}){:got_connection}, CONNECTION_POOL_DEFAULTS.merge(:max_connections=>@max_size))
   end
 
   specify "#hold should increment #created_count" do
@@ -90,7 +100,7 @@ describe "A connection pool handling connections" do
     @cpool.created_count.should <= @max_size
   end
 
-  specify ":disconnection_proc option should set the disconnection proc to use" do
+  specify "database's disconnect connection method should be called when a disconnect is detected" do
     @max_size.should == 2
     proc{@cpool.hold{raise Sequel::DatabaseDisconnectError}}.should raise_error(Sequel::DatabaseDisconnectError)
     @max_size.should == 3
@@ -127,14 +137,14 @@ end
 describe "ConnectionPool#hold" do
   before do
     value = 0
-    @c = Class.new do
+    c = @c = Class.new do
       define_method(:initialize){value += 1}
       define_method(:value){value}
     end
-    @pool = Sequel::ConnectionPool.get_pool(CONNECTION_POOL_DEFAULTS){@c.new}
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call{c.new}, CONNECTION_POOL_DEFAULTS)
   end
   
-  specify "should pass the result of the connection maker proc to the supplied block" do
+  specify "shoulda use the database's connect method to get new connections" do
     res = nil
     @pool.hold {|c| res = c}
     res.should be_a_kind_of(@c)
@@ -146,7 +156,7 @@ describe "ConnectionPool#hold" do
   
   specify "should be re-entrant by the same thread" do
     cc = nil
-    @pool.hold {|c| @pool.hold {|c| @pool.hold {|c| cc = c}}}
+    @pool.hold {|c| @pool.hold {|c1| @pool.hold {|c2| cc = c2}}}
     cc.should be_a_kind_of(@c)
   end
   
@@ -158,7 +168,8 @@ end
 describe "A connection pool with a max size of 1" do
   before do
     @invoked_count = 0
-    @pool = Sequel::ConnectionPool.get_pool(CONNECTION_POOL_DEFAULTS.merge(:max_connections=>1)) {@invoked_count += 1; 'herro'}
+    icp = proc{@invoked_count += 1}
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call{icp.call; 'herro'}, CONNECTION_POOL_DEFAULTS.merge(:max_connections=>1))
   end
   
   specify "should let only one thread access the connection at any time" do
@@ -187,7 +198,7 @@ describe "A connection pool with a max size of 1" do
     q.push nil
     q1.pop
 
-    t1.should_not be_alive
+    t1.join
     t2.should be_alive
     
     c2.should == 'hello'
@@ -209,10 +220,10 @@ describe "A connection pool with a max size of 1" do
     c1, c2, c3 = nil
     @pool.hold do |c|
       c1 = c
-      @pool.hold do |c|
-        c2 = c
-        @pool.hold do |c|
-          c3 = c
+      @pool.hold do |cc2|
+        c2 = cc2
+        @pool.hold do |cc3|
+          c3 = cc3
         end
       end
     end
@@ -228,10 +239,64 @@ describe "A connection pool with a max size of 1" do
 end
 
 shared_examples_for "A threaded connection pool" do  
-  specify "should raise a PoolTimeout error if a connection couldn't be acquired before timeout" do
-    x = nil
+  specify "should not have all_connections yield connections allocated to other threads" do
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call(&@icpp), @cp_opts.merge(:max_connections=>2, :pool_timeout=>0))
     q, q1 = Queue.new, Queue.new
-    pool = Sequel::ConnectionPool.get_pool(@cp_opts.merge(:max_connections=>1, :pool_timeout=>0)) {@invoked_count += 1}
+    t = Thread.new do
+      pool.hold do |c1|
+        q1.push nil
+        q.pop
+      end
+    end
+    pool.hold do |c1|
+      q1.pop
+      pool.all_connections{|c| c.should == c1}
+      q.push nil
+    end
+    t.join
+  end
+
+  specify "should wait until a connection is available if all are checked out" do
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call(&@icpp), @cp_opts.merge(:max_connections=>1, :pool_timeout=>0.1, :pool_sleep_time=>0))
+    q, q1 = Queue.new, Queue.new
+    t = Thread.new do
+      pool.hold do |c|
+        q1.push nil
+        3.times{Thread.pass}
+        q.pop
+      end
+    end
+    q1.pop
+    proc{pool.hold{}}.should raise_error(Sequel::PoolTimeout)
+    q.push nil
+    t.join
+  end
+
+  specify "should not have all_connections yield all available connections" do
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call(&@icpp), @cp_opts.merge(:max_connections=>2, :pool_timeout=>0))
+    q, q1 = Queue.new, Queue.new
+    b = []
+    t = Thread.new do
+      pool.hold do |c1|
+        b << c1
+        q1.push nil
+        q.pop
+      end
+    end
+    pool.hold do |c1|
+      q1.pop
+      b << c1
+      q.push nil
+    end
+    t.join
+    a = []
+    pool.all_connections{|c| a << c}
+    a.sort.should == b.sort
+  end
+
+  specify "should raise a PoolTimeout error if a connection couldn't be acquired before timeout" do
+    q, q1 = Queue.new, Queue.new
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call(&@icpp), @cp_opts.merge(:max_connections=>1, :pool_timeout=>0))
     t = Thread.new{pool.hold{|c| q1.push nil; q.pop}}
     q1.pop
     proc{pool.hold{|c|}}.should raise_error(Sequel::PoolTimeout)
@@ -240,7 +305,7 @@ shared_examples_for "A threaded connection pool" do
   end
   
   it "should not add a disconnected connection back to the pool if the disconnection_proc raises an error" do
-    pool = Sequel::ConnectionPool.get_pool(@cp_opts.merge(:max_connections=>1, :pool_timeout=>0, :disconnection_proc=>proc{|c| raise Sequel::Error})) {@invoked_count += 1}
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call(proc{|c| raise Sequel::Error}, &@icpp), @cp_opts.merge(:max_connections=>1, :pool_timeout=>0))
     proc{pool.hold{raise Sequel::DatabaseDisconnectError}}.should raise_error(Sequel::Error)
     pool.available_connections.length.should == 0
   end
@@ -301,13 +366,62 @@ shared_examples_for "A threaded connection pool" do
     @pool.available_connections.size.should == 5
     @pool.allocated.should be_empty
   end
+
+  specify "should store connections in a stack if :connection_handling=>:stack" do
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call(&@icpp), @cp_opts.merge(:connection_handling=>:stack))
+    c2 = nil
+    c = @pool.hold{|cc| Thread.new{@pool.hold{|cc2| c2 = cc2}}.join; cc}
+    @pool.size.should == 2
+    @pool.hold{|cc| cc.should == c}
+    @pool.hold{|cc| cc.should == c}
+    @pool.hold do |cc|
+      cc.should == c
+      Thread.new{@pool.hold{|cc2| cc2.should == c2}}
+    end
+  end
+
+  specify "should store connections in a queue if :connection_handling=>:queue" do
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call(&@icpp), @cp_opts.merge(:connection_handling=>:queue))
+    c2 = nil
+    c = @pool.hold{|cc| Thread.new{@pool.hold{|cc2| c2 = cc2}}.join; cc}
+    @pool.size.should == 2
+    @pool.hold{|cc| cc.should == c2}
+    @pool.hold{|cc| cc.should == c}
+    @pool.hold do |cc|
+      cc.should == c2
+      Thread.new{@pool.hold{|cc2| cc2.should == c}}
+    end
+  end
+
+  specify "should not store connections if :connection_handling=>:disconnect" do
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call(&@icpp), @cp_opts.merge(:connection_handling=>:disconnect))
+    d = []
+    meta_def(@pool.db, :disconnect_connection){|c| d << c}
+    @pool.hold do |cc|
+      cc.should == 1
+      Thread.new{@pool.hold{|cc2| cc2.should == 2}}.join
+      d.should == [2]
+      @pool.hold{|cc3| cc3.should == 1}
+    end
+    @pool.size.should == 0
+    d.should == [2, 1]
+
+    @pool.hold{|cc| cc.should == 3}
+    @pool.size.should == 0
+    d.should == [2, 1, 3]
+
+    @pool.hold{|cc| cc.should == 4}
+    @pool.size.should == 0
+    d.should == [2, 1, 3, 4]
+  end
 end
 
 describe "Threaded Unsharded Connection Pool" do
   before do
     @invoked_count = 0
+    @icpp = proc{@invoked_count += 1}
     @cp_opts = CONNECTION_POOL_DEFAULTS.merge(:max_connections=>5)
-    @pool = Sequel::ConnectionPool.get_pool(@cp_opts) {@invoked_count += 1}
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call(&@icpp), @cp_opts)
   end
   
   it_should_behave_like "A threaded connection pool"
@@ -316,8 +430,9 @@ end
 describe "Threaded Sharded Connection Pool" do
   before do
     @invoked_count = 0
+    @icpp = proc{@invoked_count += 1}
     @cp_opts = CONNECTION_POOL_DEFAULTS.merge(:max_connections=>5, :servers=>{})
-    @pool = Sequel::ConnectionPool.get_pool(@cp_opts) {@invoked_count += 1}
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call(&@icpp), @cp_opts)
   end
 
   it_should_behave_like "A threaded connection pool"
@@ -326,7 +441,8 @@ end
 describe "ConnectionPool#disconnect" do
   before do
     @count = 0
-    @pool = Sequel::ConnectionPool.get_pool(CONNECTION_POOL_DEFAULTS.merge(:max_connections=>5, :servers=>{})) {{:id => @count += 1}}
+    cp = proc{@count += 1}
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call{{:id => cp.call}}, CONNECTION_POOL_DEFAULTS.merge(:max_connections=>5, :servers=>{}))
     threads = []
     q, q1 = Queue.new, Queue.new
     5.times {|i| threads << Thread.new {@pool.hold {|c| q1.push nil; q.pop}}}
@@ -340,7 +456,8 @@ describe "ConnectionPool#disconnect" do
     @pool.available_connections.size.should == 5
     @pool.available_connections.each {|c| c[:id].should_not be_nil}
     conns = []
-    @pool.disconnect {|c| conns << c}
+    meta_def(@pool.db, :disconnect_connection){|c| conns << c}
+    @pool.disconnect
     conns.size.should == 5
   end
   
@@ -356,7 +473,8 @@ describe "ConnectionPool#disconnect" do
       @pool.available_connections.size.should == 4
       @pool.available_connections.each {|c| c.should_not be(conn)}
       conns = []
-      @pool.disconnect {|c| conns << c}
+      meta_def(@pool.db, :disconnect_connection){|c| conns << c}
+      @pool.disconnect
       conns.size.should == 4
       @pool.size.should == 1
     end
@@ -366,8 +484,18 @@ end
 
 describe "A connection pool with multiple servers" do
   before do
-    @invoked_counts = Hash.new(0)
-    @pool = Sequel::ConnectionPool.get_pool(CONNECTION_POOL_DEFAULTS.merge(:servers=>{:read_only=>{}})){|server| "#{server}#{@invoked_counts[server] += 1}"}
+    ic = @invoked_counts = Hash.new(0)
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call{|server| "#{server}#{ic[server] += 1}"}, CONNECTION_POOL_DEFAULTS.merge(:servers=>{:read_only=>{}}))
+  end
+  
+  specify "#all_connections should return connections for all servers" do
+    @pool.hold{}
+    @pool.all_connections{|c1| c1.should == "default1"}
+    a = []
+    @pool.hold(:read_only) do |c|
+      @pool.all_connections{|c1| a << c1}
+    end
+    a.sort_by{|c| c.to_s}.should == ["default1", "read_only1"]
   end
   
   specify "#servers should return symbols for all servers" do
@@ -398,7 +526,8 @@ describe "A connection pool with multiple servers" do
   end
 
   specify "should support a :servers_hash option used for converting the server argument" do
-    @pool = Sequel::ConnectionPool.get_pool(CONNECTION_POOL_DEFAULTS.merge(:servers_hash=>Hash.new(:read_only), :servers=>{:read_only=>{}})){|server| "#{server}#{@invoked_counts[server] += 1}"}
+    ic = @invoked_counts
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call{|server| "#{server}#{ic[server] += 1}"}, CONNECTION_POOL_DEFAULTS.merge(:servers_hash=>Hash.new(:read_only), :servers=>{:read_only=>{}}))
     @pool.hold(:blah) do |c1|
       c1.should == "read_only1"
       @pool.hold(:blah) do |c2|
@@ -409,7 +538,7 @@ describe "A connection pool with multiple servers" do
       end
     end
 
-    @pool = Sequel::ConnectionPool.get_pool(CONNECTION_POOL_DEFAULTS.merge(:servers_hash=>Hash.new{|h,k| raise Sequel::Error}, :servers=>{:read_only=>{}})){|server| "#{server}#{@invoked_counts[server] += 1}"}
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call{|server| "#{server}#{ic[server] += 1}"}, CONNECTION_POOL_DEFAULTS.merge(:servers_hash=>Hash.new{|h,k| raise Sequel::Error}, :servers=>{:read_only=>{}}))
     proc{@pool.hold(:blah){|c1|}}.should raise_error(Sequel::Error)
   end
 
@@ -446,7 +575,8 @@ describe "A connection pool with multiple servers" do
     conns = []
     @pool.size.should == 1
     @pool.size(:read_only).should == 1
-    @pool.disconnect{|c| conns << c}
+    meta_def(@pool.db, :disconnect_connection){|c| conns << c}
+    @pool.disconnect
     conns.sort.should == %w'default1 read_only1'
     @pool.size.should == 0
     @pool.size(:read_only).should == 0
@@ -455,7 +585,7 @@ describe "A connection pool with multiple servers" do
   end
   
   specify "#add_servers should add new servers to the pool" do
-    pool = Sequel::ConnectionPool.get_pool(:servers=>{:server1=>{}}){|s| s}
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call{|s| s}, :servers=>{:server1=>{}})
     
     pool.hold{}
     pool.hold(:server2){}
@@ -486,7 +616,7 @@ describe "A connection pool with multiple servers" do
   end
 
   specify "#add_servers should ignore existing keys" do
-    pool = Sequel::ConnectionPool.get_pool(:servers=>{:server1=>{}}){|s| s}
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call{|s| s}, :servers=>{:server1=>{}})
     
     pool.allocated.length.should == 0
     pool.allocated(:server1).length.should == 0
@@ -532,7 +662,7 @@ describe "A connection pool with multiple servers" do
   end
   
   specify "#remove_servers should disconnect available connections immediately" do
-    pool = Sequel::ConnectionPool.get_pool(:max_connections=>5, :servers=>{:server1=>{}}){|s| s}
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call{|s| s}, :max_connections=>5, :servers=>{:server1=>{}})
     threads = []
     q, q1 = Queue.new, Queue.new
     5.times {|i| threads << Thread.new {pool.hold(:server1){|c| q1.push nil; q.pop}}}
@@ -547,7 +677,7 @@ describe "A connection pool with multiple servers" do
   
   specify "#remove_servers should disconnect connections in use as soon as they are returned to the pool" do
     dc = []
-    pool = Sequel::ConnectionPool.get_pool(:servers=>{:server1=>{}}, :disconnection_proc=>proc{|c| dc << c}){|s| s}
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call(proc{|c| dc << c}){|c| c}, :servers=>{:server1=>{}})
     c1 = nil
     pool.hold(:server1) do |c|
       pool.size(:server1).should == 1
@@ -562,7 +692,7 @@ describe "A connection pool with multiple servers" do
   end
   
   specify "#remove_servers should remove server related data structures immediately" do
-    pool = Sequel::ConnectionPool.get_pool(:servers=>{:server1=>{}}){|s| s}
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call{|s| s}, :servers=>{:server1=>{}})
     pool.available_connections(:server1).should == []
     pool.allocated(:server1).should == {}
     pool.remove_servers([:server1])
@@ -571,14 +701,14 @@ describe "A connection pool with multiple servers" do
   end
   
   specify "#remove_servers should not allow the removal of the default server" do
-    pool = Sequel::ConnectionPool.get_pool(:servers=>{:server1=>{}}){|s| s}
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call{|s| s}, :servers=>{:server1=>{}})
     proc{pool.remove_servers([:server1])}.should_not raise_error
     proc{pool.remove_servers([:default])}.should raise_error(Sequel::Error)
   end
   
   specify "#remove_servers should ignore servers that have already been removed" do
     dc = []
-    pool = Sequel::ConnectionPool.get_pool(:servers=>{:server1=>{}}, :disconnection_proc=>proc{|c| dc << c}){|s| s}
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call(proc{|c| dc << c}){|c| c}, :servers=>{:server1=>{}})
     c1 = nil
     pool.hold(:server1) do |c|
       pool.size(:server1).should == 1
@@ -598,7 +728,7 @@ ST_CONNECTION_POOL_DEFAULTS = CONNECTION_POOL_DEFAULTS.merge(:single_threaded=>t
 
 describe "SingleConnectionPool" do
   before do
-    @pool = Sequel::ConnectionPool.get_pool(ST_CONNECTION_POOL_DEFAULTS){1234}
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call{1234}, ST_CONNECTION_POOL_DEFAULTS)
   end
   
   specify "should provide a #hold method" do
@@ -610,7 +740,7 @@ describe "SingleConnectionPool" do
   specify "should provide a #disconnect method" do
     conn = nil
     x = nil
-    pool = Sequel::ConnectionPool.get_pool(ST_CONNECTION_POOL_DEFAULTS.merge(:disconnection_proc=>proc{|c| conn = c})){1234}
+    pool = Sequel::ConnectionPool.get_pool(mock_db.call(proc{|c| conn = c}){1234}, ST_CONNECTION_POOL_DEFAULTS)
     pool.hold{|c| x = c}
     x.should == 1234
     pool.disconnect
@@ -621,7 +751,18 @@ end
 describe "A single threaded pool with multiple servers" do
   before do
     @max_size=2
-    @pool = Sequel::ConnectionPool.get_pool(ST_CONNECTION_POOL_DEFAULTS.merge(:disconnection_proc=>proc{|c| @max_size=3}, :servers=>{:read_only=>{}})){|server| server}
+    msp = proc{@max_size += 1}
+    @pool = Sequel::ConnectionPool.get_pool(mock_db.call(proc{|c| msp.call}){|c| c}, ST_CONNECTION_POOL_DEFAULTS.merge(:servers=>{:read_only=>{}}))
+  end
+  
+  specify "#all_connections should return connections for all servers" do
+    @pool.hold{}
+    @pool.all_connections{|c1| c1.should == :default}
+    a = []
+    @pool.hold(:read_only) do
+      @pool.all_connections{|c1| a << c1}
+    end
+    a.sort_by{|c| c.to_s}.should == [:default, :read_only]
   end
   
   specify "#servers should return symbols for all servers" do
@@ -705,11 +846,10 @@ describe "A single threaded pool with multiple servers" do
   specify "#disconnect should disconnect from all servers" do
     @pool.hold(:read_only){}
     @pool.hold{}
-    conns = []
     @pool.conn.should == :default
     @pool.conn(:read_only).should == :read_only
-    @pool.disconnect{|c| conns << c}
-    conns.sort_by{|x| x.to_s}.should == [:default, :read_only]
+    @pool.disconnect
+    @max_size.should == 4
     @pool.conn.should == nil
     @pool.conn(:read_only).should == nil
   end
@@ -730,66 +870,81 @@ describe "A single threaded pool with multiple servers" do
 end
 
 shared_examples_for "All connection pools classes" do
+  specify "should have pool_type return a symbol" do
+    @class.new(mock_db.call{123}, {}).pool_type.should be_a_kind_of(Symbol)
+  end
+
+  specify "should have all_connections yield current and available connections" do
+    p = @class.new(mock_db.call{123}, {})
+    p.hold{|c| p.all_connections{|c1| c.should == c1}}
+  end
+
+  specify "should be able to modify after_connect proc after the pool is created" do
+    a = []
+    p = @class.new(mock_db.call{123}, {})
+    p.after_connect = pr = proc{|c| a << c}
+    p.after_connect.should == pr
+    a.should == []
+    p.hold{}
+    a.should == [123]
+  end
+
   specify "should not raise an error when disconnecting twice" do
-    c = @class.new({}){123}
+    c = @class.new(mock_db.call{123}, {})
     proc{c.disconnect}.should_not raise_error
     proc{c.disconnect}.should_not raise_error
   end
   
   specify "should yield a connection created by the initialize block to hold" do
     x = nil
-    @class.new({}){123}.hold{|c| x = c}
+    @class.new(mock_db.call{123}, {}).hold{|c| x = c}
     x.should == 123
   end
   
   specify "should have the initialize block accept a shard/server argument" do
     x = nil
-    @class.new({}){|c| [c, c]}.hold{|c| x = c}
+    @class.new(mock_db.call{|c| [c, c]}, {}).hold{|c| x = c}
     x.should == [:default, :default]
   end
   
   specify "should have respect an :after_connect proc that is called with each newly created connection" do
     x = nil
-    @class.new(:after_connect=>proc{|c| x = [c, c]}){|c| 123}.hold{}
+    @class.new(mock_db.call{123}, :after_connect=>proc{|c| x = [c, c]}).hold{}
     x.should == [123, 123]
+    @class.new(mock_db.call{123}, :after_connect=>lambda{|c| x = [c, c]}).hold{}
+    x.should == [123, 123]
+    @class.new(mock_db.call{123}, :after_connect=>proc{|c, s| x = [c, s]}).hold{}
+    x.should == [123, :default]
+    @class.new(mock_db.call{123}, :after_connect=>lambda{|c, s| x = [c, s]}).hold{}
+    x.should == [123, :default]
   end
   
   specify "should raise a DatabaseConnectionError if the connection raises an exception" do
-    proc{@class.new({}){|c| raise Exception}.hold{}}.should raise_error(Sequel::DatabaseConnectionError)
+    proc{@class.new(mock_db.call{|c| raise Exception}, {}).hold{}}.should raise_error(Sequel::DatabaseConnectionError)
   end
   
   specify "should raise a DatabaseConnectionError if the initialize block returns nil" do
-    proc{@class.new({}){}.hold{}}.should raise_error(Sequel::DatabaseConnectionError)
+    proc{@class.new(mock_db.call{}, {}).hold{}}.should raise_error(Sequel::DatabaseConnectionError)
   end
   
   specify "should call the disconnection_proc option if the hold block raises a DatabaseDisconnectError" do
     x = nil
-    proc{@class.new(:disconnection_proc=>proc{|c| x = c}){123}.hold{raise Sequel::DatabaseDisconnectError}}.should raise_error(Sequel::DatabaseDisconnectError)
+    proc{@class.new(mock_db.call(proc{|c| x = c}){123}).hold{raise Sequel::DatabaseDisconnectError}}.should raise_error(Sequel::DatabaseDisconnectError)
     x.should == 123
   end
   
-  specify "should have a disconnect method that calls the :disconnection_proc option with the connection" do
+  specify "should have a disconnect method that disconnects the connection" do
     x = nil
-    c = @class.new(:disconnection_proc=>proc{|c| x = c}){123}
+    c = @class.new(mock_db.call(proc{|c1| x = c1}){123})
     c.hold{}
     x.should == nil
     c.disconnect
     x.should == 123
   end
   
-  specify "should have a disconnect method that calls the given block with the connection" do
-    x = nil
-    y = nil
-    c = @class.new(:disconnection_proc=>proc{|c| x = c}){123}
-    c.hold{}
-    c.disconnect{|c| y = c}
-    x.should == nil
-    y.should == 123
-  end
-  
   specify "should have a reentrent hold method" do
     o = Object.new
-    c = @class.new({}){o}
+    c = @class.new(mock_db.call{o}, {})
     c.hold do |x|
       x.should == o
       c.hold do |x1|
@@ -802,11 +957,11 @@ shared_examples_for "All connection pools classes" do
   end
   
   specify "should have a servers method that returns an array of shard/server symbols" do
-    @class.new({}){123}.servers.should == [:default]
+    @class.new(mock_db.call{123}, {}).servers.should == [:default]
   end
   
   specify "should have a servers method that returns an array of shard/server symbols" do
-    c = @class.new({}){123}
+    c = @class.new(mock_db.call{123}, {})
     c.size.should == 0
     c.hold{}
     c.size.should == 1
@@ -816,6 +971,9 @@ end
 Sequel::ConnectionPool::CONNECTION_POOL_MAP.keys.each do |k, v|
   opts = {:single_threaded=>k, :servers=>(v ? {} : nil)}
   describe "Connection pool with #{opts.inspect}" do
+    before(:all) do
+      Sequel::ConnectionPool.send(:get_pool, mock_db.call, opts)
+    end
     before do
       @class = Sequel::ConnectionPool.send(:connection_pool_class, opts)
     end
