@@ -390,6 +390,22 @@ describe "Blockless Ruby Filters" do
   it "should handled emulated trim function" do
     @d.lit(Sequel.trim(:a)).should == 'trim(a)'
   end
+
+  it "should handled emulated function where only name is emulated" do
+    dsc = Class.new(Sequel::Dataset)
+    dsc::EMULATED_FUNCTION_MAP[:trim] = :foo
+    dsc.new(@d.db).literal(Sequel.trim(:a)).should == 'foo(a)'
+  end
+
+  it "should handled emulated function needing full emulation" do
+    dsc = Class.new(Sequel::Dataset) do
+      def emulate_function?(n) n == :trim end
+      def emulate_function_sql_append(sql, f)
+        sql << "#{f.name}FOO(lower(#{f.args.first}))"
+      end
+    end
+    dsc.new(@d.db).literal(Sequel.trim(:a)).should == 'trimFOO(lower(a))'
+  end
 end
 
 describe Sequel::SQL::VirtualRow do
@@ -423,11 +439,25 @@ describe Sequel::SQL::VirtualRow do
     @d.l{count(:*){}}.should == 'count(*)'
   end
 
+  it "should support * method on functions to raise error if function already has an argument" do
+    proc{@d.l{count(1).*}}.should raise_error(Sequel::Error)
+  end
+
+  it "should support * method on functions to use * as the argument" do
+    @d.l{count{}.*}.should == 'count(*)'
+    @d.literal(Sequel.expr{sum(1) * 2}).should == '(sum(1) * 2)'
+  end
+
   it "should treat methods with a block and a leading argument :distinct as a function call with DISTINCT and the additional method arguments" do
     @d.l{count(:distinct, column1){}}.should == 'count(DISTINCT "column1")'
     @d.l{count(:distinct, column1, column2){}}.should == 'count(DISTINCT "column1", "column2")'
   end
 
+  it "should support distinct methods on functions to use DISTINCT before the arguments" do
+    @d.l{count(column1).distinct}.should == 'count(DISTINCT "column1")'
+    @d.l{count(column1, column2).distinct}.should == 'count(DISTINCT "column1", "column2")'
+  end
+
   it "should raise an error if an unsupported argument is used with a block" do
     proc{@d.where{count(:blah){}}}.should raise_error(Sequel::Error)
   end
@@ -479,6 +509,19 @@ describe Sequel::SQL::VirtualRow do
     @d.l{count(:over, :* =>true, :partition=>a, :order=>b, :window=>:win, :frame=>:rows){}}.should == 'count(*) OVER ("win" PARTITION BY "a" ORDER BY "b" ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)'
   end
 
+  it "should support over method on functions to create window functions" do
+    @d.l{rank{}.over}.should == 'rank() OVER ()'
+    @d.l{sum(c).over(:partition=>a, :order=>b, :window=>:win, :frame=>:rows)}.should == 'sum("c") OVER ("win" PARTITION BY "a" ORDER BY "b" ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)'
+  end
+
+  it "should support over method with a Window argument" do
+    @d.l{sum(c).over(Sequel::SQL::Window.new(:partition=>a, :order=>b, :window=>:win, :frame=>:rows))}.should == 'sum("c") OVER ("win" PARTITION BY "a" ORDER BY "b" ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)'
+  end
+
+  it "should raise error if over is called on a function that already has a window " do
+    proc{@d.l{rank{}.over.over}}.should raise_error(Sequel::Error)
+  end
+
   it "should raise an error if window functions are not supported" do
     class << @d; remove_method :supports_window_functions? end
     meta_def(@d, :supports_window_functions?){false}
@@ -486,6 +529,61 @@ describe Sequel::SQL::VirtualRow do
     proc{Sequel.mock.dataset.filter{count(:over, :* =>true, :partition=>a, :order=>b, :window=>:win, :frame=>:rows){}}.sql}.should raise_error(Sequel::Error)
   end
   
+  it "should handle lateral function calls" do
+    @d.l{rank{}.lateral}.should == 'LATERAL rank()' 
+  end
+
+  it "should handle ordered-set and hypothetical-set function calls" do
+    @d.l{mode{}.within_group(:a)}.should == 'mode() WITHIN GROUP (ORDER BY "a")' 
+    @d.l{mode{}.within_group(:a, :b)}.should == 'mode() WITHIN GROUP (ORDER BY "a", "b")' 
+  end
+
+  it "should handle filtered aggregate function calls" do
+    @d.l{count{}.*.filter(:a, :b)}.should == 'count(*) FILTER (WHERE ("a" AND "b"))' 
+    @d.l{count{}.*.filter(:a=>1)}.should == 'count(*) FILTER (WHERE ("a" = 1))'
+    @d.l{count{}.*.filter{b > 1}}.should == 'count(*) FILTER (WHERE ("b" > 1))'
+    @d.l{count{}.*.filter(:a=>1){b > 1}}.should == 'count(*) FILTER (WHERE (("a" = 1) AND ("b" > 1)))'
+  end
+
+  it "should handle fitlered ordered-set and hypothetical-set function calls" do
+    @d.l{mode{}.within_group(:a).filter(:a=>1)}.should == 'mode() WITHIN GROUP (ORDER BY "a") FILTER (WHERE ("a" = 1))' 
+  end
+
+  it "should handle function calls with ordinality" do
+    @d.l{foo{}.with_ordinality}.should == 'foo() WITH ORDINALITY' 
+  end
+
+  it "should support function method on identifiers to create functions" do
+    @d.l{rank.function}.should == 'rank()' 
+    @d.l{sum.function(c)}.should == 'sum("c")'
+    @d.l{sum.function(c, 1)}.should == 'sum("c", 1)'
+  end
+
+  it "should support function method on qualified identifiers to create functions" do
+    @d.l{sch__rank.function}.should == 'sch.rank()' 
+    @d.l{sch__sum.function(c)}.should == 'sch.sum("c")'
+    @d.l{sch__sum.function(c, 1)}.should == 'sch.sum("c", 1)'
+    @d.l{Sequel.qualify(sch__sum, :x__y).function(c, 1)}.should == 'sch.sum.x.y("c", 1)'
+  end
+
+  it "should handle quoted function names" do
+    def @d.supports_quoted_function_names?; true; end
+    @d.l{rank.function}.should == '"rank"()' 
+    @d.l{sch__rank.function}.should == '"sch"."rank"()' 
+  end
+
+  it "should quote function names if a quoted function is used and database supports quoted function names" do
+    def @d.supports_quoted_function_names?; true; end
+    @d.l{rank{}.quoted}.should == '"rank"()' 
+    @d.l{sch__rank{}.quoted}.should == '"sch__rank"()' 
+  end
+
+  it "should not quote function names if an unquoted function is used" do
+    def @d.supports_quoted_function_names?; true; end
+    @d.l{rank.function.unquoted}.should == 'rank()' 
+    @d.l{sch__rank.function.unquoted}.should == 'sch.rank()' 
+  end
+
   it "should deal with classes without requiring :: prefix" do
     @d.l{date < Date.today}.should == "(\"date\" < '#{Date.today}')"
     @d.l{date < Sequel::CURRENT_DATE}.should == "(\"date\" < CURRENT_DATE)"
@@ -678,7 +776,7 @@ describe "Sequel core extension replacements" do
   end
 
   it "Sequel.& should join all arguments given with AND" do
-    l(Sequel.&(:a), "(a)")
+    l(Sequel.&(:a), "a")
     l(Sequel.&(:a, :b=>:c), "(a AND (b = c))")
     l(Sequel.&(:a, {:b=>:c}, Sequel.lit('d')), "(a AND (b = c) AND d)")
   end
@@ -688,7 +786,7 @@ describe "Sequel core extension replacements" do
   end
 
   it "Sequel.| should join all arguments given with OR" do
-    l(Sequel.|(:a), "(a)")
+    l(Sequel.|(:a), "a")
     l(Sequel.|(:a, :b=>:c), "(a OR (b = c))")
     l(Sequel.|(:a, {:b=>:c}, Sequel.lit('d')), "(a OR (b = c) OR d)")
   end
@@ -772,7 +870,7 @@ describe "Sequel core extension replacements" do
 
   it "Sequel.{+,-,*,/} should accept arguments and use the appropriate operator" do
     %w'+ - * /'.each do |op|
-      l(Sequel.send(op, 1), '(1)')
+      l(Sequel.send(op, 1), '1')
       l(Sequel.send(op, 1, 2), "(1 #{op} 2)")
       l(Sequel.send(op, 1, 2, 3), "(1 #{op} 2 #{op} 3)")
     end
@@ -903,6 +1001,22 @@ describe "Sequel::SQLTime" do
     @db.literal(Sequel::SQLTime.create(1, 2, 3)).should == "'01:02:03.000000'"
     @db.literal(Sequel::SQLTime.create(1, 2, 3, 500000)).should == "'01:02:03.500000'"
   end
+
+  specify "#to_s should include hour, minute, and second by default" do
+    Sequel::SQLTime.create(1, 2, 3).to_s.should == "01:02:03"
+    Sequel::SQLTime.create(1, 2, 3, 500000).to_s.should == "01:02:03"
+  end
+
+  specify "#to_s should handle arguments with super" do
+    t = Sequel::SQLTime.create(1, 2, 3)
+    begin
+      Time.now.to_s('%F')
+    rescue
+      proc{t.to_s('%F')}.should raise_error
+    else
+      proc{t.to_s('%F')}.should_not raise_error
+    end
+  end
 end
 
 describe "Sequel::SQL::Wrapper" do
@@ -1002,6 +1116,12 @@ describe "Sequel.delay" do
     @o._a.should == 2
   end
 
+  specify "should call the block with the current dataset if it accepts one argument" do
+    ds = Sequel.mock[:b].where(Sequel.delay{|ds| ds.first_source})
+    ds.sql.should == "SELECT * FROM b WHERE b"
+    ds.from(:c).sql.should == "SELECT * FROM c WHERE c"
+  end
+
   specify "should have the condition specifier handling respect delayed evaluations" do
     ds = Sequel.mock[:b].where(:a=>Sequel.delay{@o.b})
     ds.sql.should == "SELECT * FROM b WHERE (a IS NULL)"
@@ -1011,6 +1131,12 @@ describe "Sequel.delay" do
     ds.sql.should == "SELECT * FROM b WHERE (a IN (1, 2))"
   end
 
+  specify "should have the condition specifier handling call block with the current dataset if it accepts one argument" do
+    ds = Sequel.mock[:b].where(:a=>Sequel.delay{|ds| ds.first_source})
+    ds.sql.should == "SELECT * FROM b WHERE (a = b)"
+    ds.from(:c).sql.should == "SELECT * FROM c WHERE (a = c)"
+  end
+
   specify "should raise if called without a block" do
     proc{Sequel.delay}.should raise_error(Sequel::Error)
   end
@@ -1066,6 +1192,6 @@ end
 
 describe "Sequel core extensions" do
   specify "should have Sequel.core_extensions? be false by default" do
-    Sequel.core_extensions?.should be_false
+    Sequel.core_extensions?.should == false
   end
 end
