@@ -14,16 +14,19 @@ describe "Sequel Mock Adapter" do
   specify "should each not return any rows by default" do
     called = false
     Sequel.mock[:t].each{|r| called = true}
-    called.should be_false
+    called.should == false
   end
 
-  specify "should return 0 for update and delete by default" do
+  specify "should return 0 for update/delete/with_sql_delete/execute_dui by default" do
     Sequel.mock[:t].update(:a=>1).should == 0
     Sequel.mock[:t].delete.should == 0
+    Sequel.mock[:t].with_sql_delete('DELETE FROM t').should == 0
+    Sequel.mock.execute_dui('DELETE FROM t').should == 0
   end
 
-  specify "should return nil for insert by default" do
+  specify "should return nil for insert/execute_insert by default" do
     Sequel.mock[:t].insert(:a=>1).should be_nil
+    Sequel.mock.execute_insert('INSERT INTO a () DEFAULT VALUES').should be_nil
   end
 
   specify "should be able to set the rows returned by each using :fetch option with a single hash" do
@@ -127,8 +130,8 @@ describe "Sequel Mock Adapter" do
   end
 
   specify "should raise Error if given an invalid object to fetch" do
-    proc{Sequel.mock(:fetch=>Class.new).get(1)}.should raise_error(Sequel::Error)
-    proc{Sequel.mock(:fetch=>Object.new).get(1)}.should raise_error(Sequel::Error)
+    proc{Sequel.mock(:fetch=>Class.new).get(:a)}.should raise_error(Sequel::Error)
+    proc{Sequel.mock(:fetch=>Object.new).get(:a)}.should raise_error(Sequel::Error)
   end
 
   specify "should be able to set the number of rows modified by update and delete using :numrows option as an integer" do
@@ -296,7 +299,14 @@ describe "Sequel Mock Adapter" do
   end
 
   specify "should not quote identifiers by default" do
-    Sequel.mock.send(:quote_identifiers_default).should be_false
+    Sequel.mock.send(:quote_identifiers_default).should == false
+  end
+
+  specify "should allow overriding of server_version" do
+    db = Sequel.mock
+    db.server_version.should == nil
+    db.server_version = 80102
+    db.server_version.should == 80102
   end
 
   specify "should not have identifier input/output methods by default" do
@@ -407,25 +417,46 @@ describe "Sequel Mock Adapter" do
 
   specify "should be able to load dialects based on the database name" do
     begin
-      qi = class Sequel::Database; @@quote_identifiers; end
-      ii = class Sequel::Database; @@identifier_input_method; end
-      io = class Sequel::Database; @@identifier_output_method; end
+      qi = class Sequel::Database; @quote_identifiers; end
+      ii = class Sequel::Database; @identifier_input_method; end
+      io = class Sequel::Database; @identifier_output_method; end
       Sequel.quote_identifiers = nil
-      class Sequel::Database; @@identifier_input_method=nil; end
-      class Sequel::Database; @@identifier_output_method=nil; end
+      class Sequel::Database; @identifier_input_method=nil; end
+      class Sequel::Database; @identifier_output_method=nil; end
       Sequel.mock(:host=>'access').select(Date.new(2011, 12, 13)).sql.should == 'SELECT #2011-12-13#'
+      Sequel.mock(:host=>'cubrid').from(:a).offset(1).sql.should == 'SELECT * FROM "a" LIMIT 1,4294967295'
       Sequel.mock(:host=>'db2').select(1).sql.should == 'SELECT 1 FROM "SYSIBM"."SYSDUMMY1"'
       Sequel.mock(:host=>'firebird')[:a].distinct.limit(1, 2).sql.should == 'SELECT DISTINCT FIRST 1 SKIP 2 * FROM "A"'
       Sequel.mock(:host=>'informix')[:a].distinct.limit(1, 2).sql.should == 'SELECT SKIP 2 FIRST 1 DISTINCT * FROM A'
       Sequel.mock(:host=>'mssql')[:a].full_text_search(:b, 'c').sql.should == "SELECT * FROM [A] WHERE (CONTAINS ([B], 'c'))"
       Sequel.mock(:host=>'mysql')[:a].full_text_search(:b, 'c').sql.should == "SELECT * FROM `a` WHERE (MATCH (`b`) AGAINST ('c'))"
       Sequel.mock(:host=>'oracle')[:a].limit(1).sql.should == 'SELECT * FROM (SELECT * FROM "A") "T1" WHERE (ROWNUM <= 1)'
-      Sequel.mock(:host=>'postgres')[:a].full_text_search(:b, 'c').sql.should == "SELECT * FROM \"a\" WHERE (to_tsvector('simple', (COALESCE(\"b\", ''))) @@ to_tsquery('simple', 'c'))"
+      Sequel.mock(:host=>'postgres')[:a].full_text_search(:b, 'c').sql.should == "SELECT * FROM \"a\" WHERE (to_tsvector(CAST('simple' AS regconfig), (COALESCE(\"b\", ''))) @@ to_tsquery(CAST('simple' AS regconfig), 'c'))"
+      Sequel.mock(:host=>'sqlanywhere').from(:a).offset(1).sql.should == 'SELECT TOP 2147483647 START AT (1 + 1) * FROM "A"'
       Sequel.mock(:host=>'sqlite')[:a___b].sql.should == "SELECT * FROM `a` AS 'b'"
     ensure
       Sequel.quote_identifiers = qi
-      Sequel::Database.send(:class_variable_set, :@@identifier_input_method, ii)
-      Sequel::Database.send(:class_variable_set, :@@identifier_output_method, io)
+      Sequel::Database.send(:instance_variable_set, :@identifier_input_method, ii)
+      Sequel::Database.send(:instance_variable_set, :@identifier_output_method, io)
     end
   end
+
+  specify "should automatically set version for adapters nedding versions" do
+    Sequel.mock(:host=>'postgres').server_version.should == 90400
+    Sequel.mock(:host=>'mssql').server_version.should == 11000000
+    Sequel.mock(:host=>'mysql').server_version.should == 50617
+    Sequel.mock(:host=>'sqlite').sqlite_version.should == 30804
+  end
+
+  specify "should stub out the primary_key method for postgres" do
+    Sequel.mock(:host=>'postgres').primary_key(:t).should == :id
+  end
+
+  specify "should stub out the bound_variable_arg method for postgres" do
+    Sequel.mock(:host=>'postgres').bound_variable_arg(:t, nil).should == :t
+  end
+
+  specify "should handle creating tables on oracle" do
+    proc{Sequel.mock(:host=>'oracle').create_table(:a){String :b}}.should_not raise_error
+  end
 end
