@@ -25,7 +25,7 @@ describe "Sequel::Plugins::LazyAttributes" do
           elsif sql =~ /id = (\d)/
             [$1]
           end.map do |x|
-            if sql =~ /SELECT name FROM/
+            if sql =~ /SELECT (la.)?name FROM/
               {:name=>x.to_s}
             else
               {:id=>x.to_i, :name=>x.to_s}
@@ -46,7 +46,6 @@ describe "Sequel::Plugins::LazyAttributes" do
     @c.set_dataset(@ds.select(:id, :blah))
     @c.dataset.sql.should == 'SELECT id, blah FROM la'
     @c.plugin :lazy_attributes, :blah
-    @c.dataset.opts[:select].should == [:id]
     @c.dataset.sql.should == 'SELECT id FROM la'
   end
   
@@ -54,13 +53,18 @@ describe "Sequel::Plugins::LazyAttributes" do
     @c.set_dataset(@ds.select(:id, :blah))
     @c.dataset.sql.should == 'SELECT id, blah FROM la'
     @c.lazy_attributes :blah
-    @c.dataset.opts[:select].should == [:id]
     @c.dataset.sql.should == 'SELECT id FROM la'
   end
 
+  it "should handle lazy attributes that are qualified in the selection" do
+    @c.set_dataset(@ds.select(:la__id, :la__blah))
+    @c.dataset.sql.should == 'SELECT la.id, la.blah FROM la'
+    @c.plugin :lazy_attributes, :blah
+    @c.dataset.sql.should == 'SELECT la.id FROM la'
+  end
+  
   it "should remove the attributes given from the SELECT columns of the model's dataset" do
-    @ds.opts[:select].should == [:id]
-    @ds.sql.should == 'SELECT id FROM la'
+    @ds.sql.should == 'SELECT la.id FROM la'
   end
 
   it "should still typecast correctly in lazy loaded column setters" do
@@ -80,16 +84,16 @@ describe "Sequel::Plugins::LazyAttributes" do
     m.values.should == {:id=>1}
     m.name.should == '1'
     m.values.should == {:id=>1, :name=>'1'}
-    @db.sqls.should == ['SELECT id FROM la LIMIT 1', 'SELECT name FROM la WHERE (id = 1) LIMIT 1']
+    @db.sqls.should == ['SELECT la.id FROM la LIMIT 1', 'SELECT la.name FROM la WHERE (id = 1) LIMIT 1']
   end
 
   it "should lazily load the attribute for a frozen model object" do
     m = @c.first
     m.freeze
     m.name.should == '1'
-    @db.sqls.should == ['SELECT id FROM la LIMIT 1', 'SELECT name FROM la WHERE (id = 1) LIMIT 1']
+    @db.sqls.should == ['SELECT la.id FROM la LIMIT 1', 'SELECT la.name FROM la WHERE (id = 1) LIMIT 1']
     m.name.should == '1'
-    @db.sqls.should == ['SELECT name FROM la WHERE (id = 1) LIMIT 1']
+    @db.sqls.should == ['SELECT la.name FROM la WHERE (id = 1) LIMIT 1']
   end
 
   it "should not lazily load the attribute for a single model object if the value already exists" do
@@ -98,7 +102,7 @@ describe "Sequel::Plugins::LazyAttributes" do
     m[:name] = '1'
     m.name.should == '1'
     m.values.should == {:id=>1, :name=>'1'}
-    @db.sqls.should == ['SELECT id FROM la LIMIT 1']
+    @db.sqls.should == ['SELECT la.id FROM la LIMIT 1']
   end
 
   it "should not lazily load the attribute for a single model object if it is a new record" do
@@ -114,16 +118,16 @@ describe "Sequel::Plugins::LazyAttributes" do
     ms.map{|m| m.name}.should == %w'1 2'
     ms.map{|m| m.values}.should == [{:id=>1, :name=>'1'}, {:id=>2, :name=>'2'}]
     sqls = @db.sqls
-    ['SELECT id, name FROM la WHERE (id IN (1, 2))',
-     'SELECT id, name FROM la WHERE (id IN (2, 1))'].should include(sqls.pop)
-    sqls.should == ['SELECT id FROM la']
+    ['SELECT la.id, la.name FROM la WHERE (la.id IN (1, 2))',
+     'SELECT la.id, la.name FROM la WHERE (la.id IN (2, 1))'].should include(sqls.pop)
+    sqls.should == ['SELECT la.id FROM la']
   end
 
   it "should not eagerly load the attribute if model instance is frozen, and deal with other frozen instances if not frozen" do
     ms = @c.all
     ms.first.freeze
     ms.map{|m| m.name}.should == %w'1 2'
-    @db.sqls.should == ['SELECT id FROM la', 'SELECT name FROM la WHERE (id = 1) LIMIT 1', 'SELECT id, name FROM la WHERE (id IN (2))']
+    @db.sqls.should == ['SELECT la.id FROM la', 'SELECT la.name FROM la WHERE (id = 1) LIMIT 1', 'SELECT la.id, la.name FROM la WHERE (la.id IN (2))']
   end
 
   it "should add the accessors to a module included in the class, so they can be easily overridden" do
@@ -137,9 +141,9 @@ describe "Sequel::Plugins::LazyAttributes" do
     ms.map{|m| m.name}.should == %w'1-blah 2-blah'
     ms.map{|m| m.values}.should == [{:id=>1, :name=>'1'}, {:id=>2, :name=>'2'}]
     sqls = @db.sqls
-    ['SELECT id, name FROM la WHERE (id IN (1, 2))',
-     'SELECT id, name FROM la WHERE (id IN (2, 1))'].should include(sqls.pop)
-    sqls.should == ['SELECT id FROM la']
+    ['SELECT la.id, la.name FROM la WHERE (la.id IN (1, 2))',
+     'SELECT la.id, la.name FROM la WHERE (la.id IN (2, 1))'].should include(sqls.pop)
+    sqls.should == ['SELECT la.id FROM la']
   end
 
   it "should work with the serialization plugin" do
@@ -152,15 +156,15 @@ describe "Sequel::Plugins::LazyAttributes" do
     ms.map{|m| m.deserialized_values}.should == [{:name=>3}, {:name=>6}]
     ms.map{|m| m.name}.should == [3,6]
     sqls = @db.sqls
-    ['SELECT id, name FROM la WHERE (id IN (1, 2))',
-     'SELECT id, name FROM la WHERE (id IN (2, 1))'].should include(sqls.pop)
-    sqls.should == ['SELECT id FROM la']
+    ['SELECT la.id, la.name FROM la WHERE (la.id IN (1, 2))',
+     'SELECT la.id, la.name FROM la WHERE (la.id IN (2, 1))'].should include(sqls.pop)
+    sqls.should == ['SELECT la.id FROM la']
     m = @ds.first
     m.values.should == {:id=>1}
     m.name.should == 3
     m.values.should == {:id=>1, :name=>"--- 3\n"}
     m.deserialized_values.should == {:name=>3}
     m.name.should == 3
-    @db.sqls.should == ["SELECT id FROM la LIMIT 1", "SELECT name FROM la WHERE (id = 1) LIMIT 1"]
+    @db.sqls.should == ["SELECT la.id FROM la LIMIT 1", "SELECT la.name FROM la WHERE (id = 1) LIMIT 1"]
   end
 end
