@@ -2,7 +2,7 @@ require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper')
 
 describe "Sequel::Schema::Generator dump methods" do
   before do
-    @d = Sequel::Database.new
+    @d = Sequel::Database.new.extension(:schema_dumper)
     @g = Sequel::Schema::Generator
   end
 
@@ -63,11 +63,11 @@ end
 
 describe "Sequel::Database dump methods" do
   before do
-    @d = Sequel::Database.new
+    @d = Sequel::Database.new.extension(:schema_dumper)
     @d.meta_def(:tables){|o| [:t1, :t2]}
     @d.meta_def(:schema) do |t, *o|
       case t
-      when :t1, 't__t1', :t__t1.identifier
+      when :t1, 't__t1', Sequel.identifier(:t__t1)
         [[:c1, {:db_type=>'integer', :primary_key=>true, :allow_null=>false}],
          [:c2, {:db_type=>'varchar(20)', :allow_null=>true}]]
       when :t2
@@ -88,7 +88,7 @@ describe "Sequel::Database dump methods" do
   end
 
   it "should support dumping table schemas when given an identifier" do
-    @d.dump_table_schema(:t__t1.identifier).should == "create_table(\"t__t1\") do\n  primary_key :c1\n  String :c2, :size=>20\nend"
+    @d.dump_table_schema(Sequel.identifier(:t__t1)).should == "create_table(\"t__t1\") do\n  primary_key :c1\n  String :c2, :size=>20\nend"
   end
 
   it "should dump non-Integer primary key columns with explicit :type" do
@@ -98,12 +98,14 @@ describe "Sequel::Database dump methods" do
 
   it "should handle foreign keys" do
     @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :allow_null=>true}]]}
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2]}]}
     @d.dump_table_schema(:t6).should == "create_table(:t6) do\n  foreign_key :c1, :t2, :key=>[:c2]\nend"
   end
 
   it "should handle primary keys that are also foreign keys" do
     @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :primary_key=>true, :allow_null=>true}]]}
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2]}]}
     s = @d.dump_table_schema(:t6)
     s.should =~ /create_table\(:t6\) do\n  primary_key :c1, /
@@ -113,6 +115,7 @@ describe "Sequel::Database dump methods" do
 
   it "should handle foreign key options" do
     @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :allow_null=>true}]]}
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:restrict, :on_update=>:set_null, :deferrable=>true}]}
     s = @d.dump_table_schema(:t6)
     s.should =~ /create_table\(:t6\) do\n  foreign_key :c1, :t2, /
@@ -124,6 +127,7 @@ describe "Sequel::Database dump methods" do
 
   it "should handle foreign key options in the primary key" do
     @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :primary_key=>true, :allow_null=>true}]]}
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:restrict, :on_update=>:set_null, :deferrable=>true}]}
     s = @d.dump_table_schema(:t6)
     s.should =~ /create_table\(:t6\) do\n  primary_key :c1, /
@@ -136,6 +140,7 @@ describe "Sequel::Database dump methods" do
 
   it "should omit foreign key options that are the same as defaults" do
     @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :allow_null=>true}]]}
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:no_action, :on_update=>:no_action, :deferrable=>false}]}
     s = @d.dump_table_schema(:t6)
     s.should =~ /create_table\(:t6\) do\n  foreign_key :c1, :t2, /
@@ -147,6 +152,7 @@ describe "Sequel::Database dump methods" do
 
   it "should omit foreign key options that are the same as defaults in the primary key" do
     @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :primary_key=>true, :allow_null=>true}]]}
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:no_action, :on_update=>:no_action, :deferrable=>false}]}
     s = @d.dump_table_schema(:t6)
     s.should =~ /create_table\(:t6\) do\n  primary_key :c1, /
@@ -157,9 +163,9 @@ describe "Sequel::Database dump methods" do
     s.should_not =~ /:deferrable/
   end
 
-  it "should dump primary key columns with explicit :type equal to the database type when :same_db option is passed" do
+  it "should dump primary key columns with explicit type equal to the database type when :same_db option is passed" do
     @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'somedbspecifictype', :primary_key=>true, :allow_null=>false}]]}
-    @d.dump_table_schema(:t7, :same_db => true).should == "create_table(:t7) do\n  primary_key :c1, :type=>\"somedbspecifictype\"\nend"
+    @d.dump_table_schema(:t7, :same_db => true).should == "create_table(:t7) do\n  column :c1, \"somedbspecifictype\", :null=>false\n  \n  primary_key [:c1]\nend"
   end
 
   it "should use a composite primary_key calls if there is a composite primary key" do
@@ -168,11 +174,13 @@ describe "Sequel::Database dump methods" do
 
   it "should use a composite foreign_key calls if there is a composite foreign key" do
     @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer'}], [:c2, {:db_type=>'integer'}]]}
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1, :c2], :table=>:t2, :key=>[:c3, :c4]}]}
     @d.dump_table_schema(:t1).should == "create_table(:t1) do\n  Integer :c1\n  Integer :c2\n  \n  foreign_key [:c1, :c2], :t2, :key=>[:c3, :c4]\nend"
   end
 
   it "should include index information if available" do
+    @d.meta_def(:supports_index_parsing?){true}
     @d.meta_def(:indexes) do |t|
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true}}
@@ -183,7 +191,7 @@ describe "Sequel::Database dump methods" do
   it "should support dumping the whole database as a migration" do
     @d.dump_schema_migration.should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     create_table(:t1) do
       primary_key :c1
       String :c2, :size=>20
@@ -196,10 +204,6 @@ Sequel.migration do
       primary_key [:c1, :c2]
     end
   end
-  
-  down do
-    drop_table(:t2, :t1)
-  end
 end
 END_MIG
   end
@@ -208,7 +212,7 @@ END_MIG
     @d.meta_def(:tables){|o| [:t2, :t1]}
     @d.dump_schema_migration.should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     create_table(:t1) do
       primary_key :c1
       String :c2, :size=>20
@@ -221,10 +225,6 @@ Sequel.migration do
       primary_key [:c1, :c2]
     end
   end
-  
-  down do
-    drop_table(:t2, :t1)
-  end
 end
 END_MIG
   end
@@ -234,12 +234,13 @@ END_MIG
     @d.meta_def(:schema) do |t|
       t == :t1 ? [[:c2, {:db_type=>'integer'}]] : [[:c1, {:db_type=>'integer', :primary_key=>true}]]
     end
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list) do |t|
       t == :t1 ? [{:columns=>[:c2], :table=>:t2, :key=>[:c1]}] : []
     end
     @d.dump_schema_migration.should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     create_table(:t2) do
       primary_key :c1
     end
@@ -248,10 +249,6 @@ Sequel.migration do
       foreign_key :c2, :t2, :key=>[:c1]
     end
   end
-  
-  down do
-    drop_table(:t1, :t2)
-  end
 end
 END_MIG
   end
@@ -261,12 +258,13 @@ END_MIG
     @d.meta_def(:schema) do |t|
       t == :t1 ? [[:c2, {:db_type=>'integer'}]] : [[:c1, {:db_type=>'integer'}]]
     end
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list) do |t|
       t == :t1 ? [{:columns=>[:c2], :table=>:t2, :key=>[:c1]}] : [{:columns=>[:c1], :table=>:t1, :key=>[:c2]}]
     end
     @d.dump_schema_migration.should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     create_table(:t1) do
       Integer :c2
     end
@@ -279,10 +277,6 @@ Sequel.migration do
       add_foreign_key [:c2], :t2, :key=>[:c1]
     end
   end
-  
-  down do
-    drop_table(:t2, :t1)
-  end
 end
 END_MIG
   end
@@ -292,13 +286,14 @@ END_MIG
     @d.meta_def(:schema) do |t|
       t == :t1 ? [[:c2, {:db_type=>'integer'}]] : [[:c1, {:db_type=>'integer', :primary_key=>true}]]
     end
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list) do |t|
       raise Sequel::DatabaseError unless [:t1, :t2].include?(t)
       t == :t1 ? [{:columns=>[:c2], :table=>:t2, :key=>[:c1]}] : []
     end
     @d.dump_schema_migration.should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     create_table(:t2) do
       primary_key :c1
     end
@@ -307,10 +302,6 @@ Sequel.migration do
       foreign_key :c2, :t2, :key=>[:c1]
     end
   end
-  
-  down do
-    drop_table(:t1, :t2)
-  end
 end
 END_MIG
   end
@@ -319,7 +310,7 @@ END_MIG
     @d.dump_table_schema(:t1, :same_db=>true).should == "create_table(:t1) do\n  primary_key :c1\n  column :c2, \"varchar(20)\"\nend"
     @d.dump_schema_migration(:same_db=>true).should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     create_table(:t1) do
       primary_key :c1
       column :c2, "varchar(20)"
@@ -332,15 +323,12 @@ Sequel.migration do
       primary_key [:c1, :c2]
     end
   end
-  
-  down do
-    drop_table(:t2, :t1)
-  end
 end
 END_MIG
   end
 
   it "should honor the :index_names => false option to not include names of indexes" do
+    @d.meta_def(:supports_index_parsing?){true}
     @d.meta_def(:indexes) do |t|
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true}}
@@ -348,7 +336,7 @@ END_MIG
     @d.dump_table_schema(:t1, :index_names=>false).should == "create_table(:t1, :ignore_index_errors=>true) do\n  primary_key :c1\n  String :c2, :size=>20\n  \n  index [:c1]\n  index [:c2, :c1], :unique=>true\nend"
     @d.dump_schema_migration(:index_names=>false).should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     create_table(:t1, :ignore_index_errors=>true) do
       primary_key :c1
       String :c2, :size=>20
@@ -367,15 +355,12 @@ Sequel.migration do
       index [:c2, :c1], :unique=>true
     end
   end
-  
-  down do
-    drop_table(:t2, :t1)
-  end
 end
 END_MIG
   end
   
   it "should make :index_names => :namespace option a noop if there is a  global index namespace" do
+    @d.meta_def(:supports_index_parsing?){true}
     @d.meta_def(:indexes) do |t|
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>false}}
@@ -383,7 +368,7 @@ END_MIG
     @d.dump_table_schema(:t1, :index_names=>:namespace).should == "create_table(:t1, :ignore_index_errors=>true) do\n  primary_key :c1\n  String :c2, :size=>20\n  \n  index [:c1], :name=>:i1\n  index [:c2, :c1]\nend"
     @d.dump_schema_migration(:index_names=>:namespace).should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     create_table(:t1, :ignore_index_errors=>true) do
       primary_key :c1
       String :c2, :size=>20
@@ -402,16 +387,13 @@ Sequel.migration do
       index [:c2, :c1], :name=>:t1_c2_c1_index
     end
   end
-  
-  down do
-    drop_table(:t2, :t1)
-  end
 end
 END_MIG
   end
 
   it "should honor the :index_names => :namespace option to include names of indexes with prepended table name if there is no global index namespace" do
     @d.meta_def(:global_index_namespace?){false}
+    @d.meta_def(:supports_index_parsing?){true}
     @d.meta_def(:indexes) do |t|
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>false}}
@@ -419,7 +401,7 @@ END_MIG
     @d.dump_table_schema(:t1, :index_names=>:namespace).should == "create_table(:t1, :ignore_index_errors=>true) do\n  primary_key :c1\n  String :c2, :size=>20\n  \n  index [:c1], :name=>:t1_i1\n  index [:c2, :c1]\nend"
     @d.dump_schema_migration(:index_names=>:namespace).should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     create_table(:t1, :ignore_index_errors=>true) do
       primary_key :c1
       String :c2, :size=>20
@@ -438,15 +420,12 @@ Sequel.migration do
       index [:c2, :c1], :name=>:t2_t1_c2_c1_index
     end
   end
-  
-  down do
-    drop_table(:t2, :t1)
-  end
 end
 END_MIG
   end
 
   it "should honor the :indexes => false option to not include indexes" do
+    @d.meta_def(:supports_index_parsing?){true}
     @d.meta_def(:indexes) do |t|
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true}}
@@ -454,7 +433,7 @@ END_MIG
     @d.dump_table_schema(:t1, :indexes=>false).should == "create_table(:t1) do\n  primary_key :c1\n  String :c2, :size=>20\nend"
     @d.dump_schema_migration(:indexes=>false).should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     create_table(:t1) do
       primary_key :c1
       String :c2, :size=>20
@@ -467,10 +446,6 @@ Sequel.migration do
       primary_key [:c1, :c2]
     end
   end
-  
-  down do
-    drop_table(:t2, :t1)
-  end
 end
 END_MIG
   end
@@ -483,6 +458,7 @@ END_MIG
   end
 
   it "should have :foreign_keys option override :indexes => false disabling of foreign keys" do
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list) do |t|
       t == :t1 ? [{:columns=>[:c2], :table=>:t2, :key=>[:c1]}] : []
     end
@@ -491,69 +467,54 @@ END_MIG
 
   it "should support dumping just indexes as a migration" do
     @d.meta_def(:tables){|o| [:t1]}
+    @d.meta_def(:supports_index_parsing?){true}
     @d.meta_def(:indexes) do |t|
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true}}
     end
     @d.dump_indexes_migration.should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     add_index :t1, [:c1], :ignore_errors=>true, :name=>:i1
     add_index :t1, [:c2, :c1], :ignore_errors=>true, :unique=>true
   end
-  
-  down do
-    drop_index :t1, [:c2, :c1], :ignore_errors=>true, :unique=>true
-    drop_index :t1, [:c1], :ignore_errors=>true, :name=>:i1
-  end
 end
 END_MIG
   end
 
   it "should honor the :index_names => false option to not include names of indexes when dumping just indexes as a migration" do
     @d.meta_def(:tables){|o| [:t1]}
+    @d.meta_def(:supports_index_parsing?){true}
     @d.meta_def(:indexes) do |t|
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true}}
     end
     @d.dump_indexes_migration(:index_names=>false).should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     add_index :t1, [:c1], :ignore_errors=>true
     add_index :t1, [:c2, :c1], :ignore_errors=>true, :unique=>true
   end
-  
-  down do
-    drop_index :t1, [:c2, :c1], :ignore_errors=>true, :unique=>true
-    drop_index :t1, [:c1], :ignore_errors=>true
-  end
 end
 END_MIG
   end
 
   it "should honor the :index_names => :namespace option be a noop if there is a global index namespace" do
     @d.meta_def(:tables){|o| [:t1, :t2]}
+    @d.meta_def(:supports_index_parsing?){true}
     @d.meta_def(:indexes) do |t|
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>false}}
     end
     @d.dump_indexes_migration(:index_names=>:namespace).should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     add_index :t1, [:c1], :ignore_errors=>true, :name=>:i1
     add_index :t1, [:c2, :c1], :ignore_errors=>true
     
     add_index :t2, [:c1], :ignore_errors=>true, :name=>:i1
     add_index :t2, [:c2, :c1], :ignore_errors=>true, :name=>:t1_c2_c1_index
   end
-  
-  down do
-    drop_index :t2, [:c2, :c1], :ignore_errors=>true, :name=>:t1_c2_c1_index
-    drop_index :t2, [:c1], :ignore_errors=>true, :name=>:i1
-    
-    drop_index :t1, [:c2, :c1], :ignore_errors=>true
-    drop_index :t1, [:c1], :ignore_errors=>true, :name=>:i1
-  end
 end
 END_MIG
   end
@@ -561,27 +522,20 @@ END_MIG
   it "should honor the :index_names => :namespace option to include names of indexes with prepended table name when dumping just indexes as a migration if there is no global index namespace" do
     @d.meta_def(:global_index_namespace?){false}
     @d.meta_def(:tables){|o| [:t1, :t2]}
+    @d.meta_def(:supports_index_parsing?){true}
     @d.meta_def(:indexes) do |t|
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>false}}
     end
     @d.dump_indexes_migration(:index_names=>:namespace).should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     add_index :t1, [:c1], :ignore_errors=>true, :name=>:t1_i1
     add_index :t1, [:c2, :c1], :ignore_errors=>true
     
     add_index :t2, [:c1], :ignore_errors=>true, :name=>:t2_i1
     add_index :t2, [:c2, :c1], :ignore_errors=>true, :name=>:t2_t1_c2_c1_index
   end
-  
-  down do
-    drop_index :t2, [:c2, :c1], :ignore_errors=>true, :name=>:t2_t1_c2_c1_index
-    drop_index :t2, [:c1], :ignore_errors=>true, :name=>:t2_i1
-    
-    drop_index :t1, [:c2, :c1], :ignore_errors=>true
-    drop_index :t1, [:c1], :ignore_errors=>true, :name=>:t1_i1
-  end
 end
 END_MIG
   end
@@ -590,11 +544,7 @@ END_MIG
     @d.meta_def(:tables){|o| [:t1]}
     @d.dump_indexes_migration.should == <<-END_MIG
 Sequel.migration do
-  up do
-    
-  end
-  
-  down do
+  change do
     
   end
 end
@@ -605,7 +555,7 @@ END_MIG
     @d.meta_def(:tables){|o| [:t1]}
     @d.dump_foreign_key_migration.should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     
   end
 end
@@ -617,6 +567,7 @@ END_MIG
     @d.meta_def(:schema) do |t|
       t == :t1 ? [[:c2, {:db_type=>'integer'}]] : [[:c1, {:db_type=>'integer'}]]
     end
+    @d.meta_def(:supports_foreign_key_parsing?){true}
     @d.meta_def(:foreign_key_list) do |t, *a|
       case t
       when :t1
@@ -629,7 +580,7 @@ END_MIG
     end
     @d.dump_foreign_key_migration.should == <<-END_MIG
 Sequel.migration do
-  up do
+  change do
     alter_table(:t1) do
       add_foreign_key [:c2], :t2, :key=>[:c1]
     end
@@ -658,22 +609,26 @@ END_MIG
        [:c7, {:db_type=>'date', :default=>"'2008-10-29'", :type=>:date, :allow_null=>true}],
        [:c8, {:db_type=>'datetime', :default=>"'2008-10-29 10:20:30'", :type=>:datetime, :allow_null=>true}],
        [:c9, {:db_type=>'time', :default=>"'10:20:30'", :type=>:time, :allow_null=>true}],
-       [:c10, {:db_type=>'interval', :default=>"'6 weeks'", :type=>:interval, :allow_null=>true}]]
+       [:c10, {:db_type=>'foo', :default=>"'6 weeks'", :type=>nil, :allow_null=>true}],
+       [:c11, {:db_type=>'date', :default=>"CURRENT_DATE", :type=>:date, :allow_null=>true}],
+       [:c12, {:db_type=>'timestamp', :default=>"now()", :type=>:datetime, :allow_null=>true}]]
       s.each{|_, c| c[:ruby_default] = column_schema_to_ruby_default(c[:default], c[:type])}
       s
     end
-    @d.dump_table_schema(:t4).gsub(/[+-]\d\d:\d\d"\)/, '")').should == "create_table(:t4) do\n  TrueClass :c1, :default=>false\n  String :c2, :default=>\"blah\"\n  Integer :c3, :default=>-1\n  Float :c4, :default=>1.0\n  BigDecimal :c5, :default=>BigDecimal.new(\"0.1005E3\")\n  File :c6, :default=>Sequel::SQL::Blob.new(\"blah\")\n  Date :c7, :default=>Date.parse(\"2008-10-29\")\n  DateTime :c8, :default=>DateTime.parse(\"2008-10-29T10:20:30\")\n  Time :c9, :default=>Sequel::SQLTime.parse(\"10:20:30\"), :only_time=>true\n  String :c10\nend"
-    @d.dump_table_schema(:t4, :same_db=>true).gsub(/[+-]\d\d:\d\d"\)/, '")').should == "create_table(:t4) do\n  column :c1, \"boolean\", :default=>false\n  column :c2, \"varchar\", :default=>\"blah\"\n  column :c3, \"integer\", :default=>-1\n  column :c4, \"float\", :default=>1.0\n  column :c5, \"decimal\", :default=>BigDecimal.new(\"0.1005E3\")\n  column :c6, \"blob\", :default=>Sequel::SQL::Blob.new(\"blah\")\n  column :c7, \"date\", :default=>Date.parse(\"2008-10-29\")\n  column :c8, \"datetime\", :default=>DateTime.parse(\"2008-10-29T10:20:30\")\n  column :c9, \"time\", :default=>Sequel::SQLTime.parse(\"10:20:30\")\n  column :c10, \"interval\", :default=>\"'6 weeks'\".lit\nend"
+    @d.dump_table_schema(:t4).gsub(/[+-]\d\d\d\d"\)/, '")').gsub(/\.0+/, '.0').should == "create_table(:t4) do\n  TrueClass :c1, :default=>false\n  String :c2, :default=>\"blah\"\n  Integer :c3, :default=>-1\n  Float :c4, :default=>1.0\n  BigDecimal :c5, :default=>BigDecimal.new(\"0.1005E3\")\n  File :c6, :default=>Sequel::SQL::Blob.new(\"blah\")\n  Date :c7, :default=>Date.new(2008, 10, 29)\n  DateTime :c8, :default=>DateTime.parse(\"2008-10-29T10:20:30.0\")\n  Time :c9, :default=>Sequel::SQLTime.parse(\"10:20:30.0\"), :only_time=>true\n  String :c10\n  Date :c11, :default=>Sequel::CURRENT_DATE\n  DateTime :c12, :default=>Sequel::CURRENT_TIMESTAMP\nend"
+    @d.dump_table_schema(:t4, :same_db=>true).gsub(/[+-]\d\d\d\d"\)/, '")').gsub(/\.0+/, '.0').should == "create_table(:t4) do\n  column :c1, \"boolean\", :default=>false\n  column :c2, \"varchar\", :default=>\"blah\"\n  column :c3, \"integer\", :default=>-1\n  column :c4, \"float\", :default=>1.0\n  column :c5, \"decimal\", :default=>BigDecimal.new(\"0.1005E3\")\n  column :c6, \"blob\", :default=>Sequel::SQL::Blob.new(\"blah\")\n  column :c7, \"date\", :default=>Date.new(2008, 10, 29)\n  column :c8, \"datetime\", :default=>DateTime.parse(\"2008-10-29T10:20:30.0\")\n  column :c9, \"time\", :default=>Sequel::SQLTime.parse(\"10:20:30.0\")\n  column :c10, \"foo\", :default=>Sequel::LiteralString.new(\"'6 weeks'\")\n  column :c11, \"date\", :default=>Sequel::CURRENT_DATE\n  column :c12, \"timestamp\", :default=>Sequel::CURRENT_TIMESTAMP\nend"
   end
   
-  it "should not use a '...'.lit as a fallback if using MySQL with the :same_db option" do
+  it "should not use a literal string as a fallback if using MySQL with the :same_db option" do
     @d.meta_def(:database_type){:mysql}
+    @d.meta_def(:supports_index_parsing?){false}
+    @d.meta_def(:supports_foreign_key_parsing?){false}
     @d.meta_def(:schema) do |t, *os|
-      s = [[:c10, {:db_type=>'interval', :default=>"'6 weeks'", :type=>:interval, :allow_null=>true}]]
+      s = [[:c10, {:db_type=>'foo', :default=>"'6 weeks'", :type=>nil, :allow_null=>true}]]
       s.each{|_, c| c[:ruby_default] = column_schema_to_ruby_default(c[:default], c[:type])}
       s
     end
-    @d.dump_table_schema(:t5, :same_db=>true).should == "create_table(:t5) do\n  column :c10, \"interval\"\nend"
+    @d.dump_table_schema(:t5, :same_db=>true).should == "create_table(:t5) do\n  column :c10, \"foo\"\nend"
   end
 
   it "should convert unknown database types to strings" do
@@ -771,7 +726,63 @@ create_table(:x) do
   Integer :c70
   Integer :c71
   Integer :c72
+  
+  check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c64), 0)
+  check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c65), 0)
+  check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c66), 0)
+  check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c67), 0)
+  check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c68), 0)
+  check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c69), 0)
 end
 END_MIG
   end
+
+  it "should convert mysql types to ruby types" do
+    types = ['double(15,2)', 'double(7,1) unsigned']
+    @d.meta_def(:schema) do |t, *o|
+      i = 0
+      types.map{|x| [:"c#{i+=1}", {:db_type=>x, :allow_null=>true}]}
+    end
+    @d.dump_table_schema(:x).should == (<<END_MIG).chomp
+create_table(:x) do
+  Float :c1
+  Float :c2
+  
+  check Sequel::SQL::BooleanExpression.new(:>=, Sequel::SQL::Identifier.new(:c2), 0)
+end
+END_MIG
+  end
+
+  it "should force specify :null option for MySQL timestamp columns when using :same_db" do
+    @d.meta_def(:database_type){:mysql}
+    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'timestamp', :primary_key=>true, :allow_null=>true}]]}
+    @d.dump_table_schema(:t3, :same_db=>true).should == "create_table(:t3) do\n  column :c1, \"timestamp\", :null=>true\n  \n  primary_key [:c1]\nend"
+
+    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'timestamp', :primary_key=>true, :allow_null=>false}]]}
+    @d.dump_table_schema(:t3, :same_db=>true).should == "create_table(:t3) do\n  column :c1, \"timestamp\", :null=>false\n  \n  primary_key [:c1]\nend"
+  end
+
+  it "should use separate primary_key call with non autoincrementable types" do
+    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'varchar(8)', :primary_key=>true}]]}
+    @d.dump_table_schema(:t3).should == "create_table(:t3) do\n  String :c1, :size=>8\n  \n  primary_key [:c1]\nend"
+    @d.dump_table_schema(:t3, :same_db=>true).should == "create_table(:t3) do\n  column :c1, \"varchar(8)\"\n  \n  primary_key [:c1]\nend"
+  end
+
+  it "should use explicit type for non integer foreign_key types" do
+    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'date', :primary_key=>true}]]}
+    @d.meta_def(:supports_foreign_key_parsing?){true}
+    @d.meta_def(:foreign_key_list){|t, *a| [{:columns=>[:c1], :table=>:t3, :key=>[:c1]}] if t == :t4}
+    ["create_table(:t4) do\n  foreign_key :c1, :t3, :type=>Date, :key=>[:c1]\n  \n  primary_key [:c1]\nend",
+     "create_table(:t4) do\n  foreign_key :c1, :t3, :key=>[:c1], :type=>Date\n  \n  primary_key [:c1]\nend"].should include(@d.dump_table_schema(:t4))
+    ["create_table(:t4) do\n  foreign_key :c1, :t3, :type=>\"date\", :key=>[:c1]\n  \n  primary_key [:c1]\nend",
+     "create_table(:t4) do\n  foreign_key :c1, :t3, :key=>[:c1], :type=>\"date\"\n  \n  primary_key [:c1]\nend"].should include(@d.dump_table_schema(:t4, :same_db=>true))
+  end
+
+  it "should correctly handing autoincrementing primary keys that are also foreign keys" do
+    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :primary_key=>true}]]}
+    @d.meta_def(:supports_foreign_key_parsing?){true}
+    @d.meta_def(:foreign_key_list){|t, *a| [{:columns=>[:c1], :table=>:t3, :key=>[:c1]}] if t == :t4}
+    ["create_table(:t4) do\n  primary_key :c1, :table=>:t3, :key=>[:c1]\nend",
+     "create_table(:t4) do\n  primary_key :c1, :key=>[:c1], :table=>:t3\nend"].should include(@d.dump_table_schema(:t4))
+  end
 end
