@@ -70,19 +70,32 @@ describe Sequel::Model, "single table inheritance plugin" do
     called.should == false
   end
 
-  it "should add a before_create hook that sets the model class name for the key" do
+  it "should set the model class name when saving" do
     StiTest.new.save
     StiTestSub1.new.save
     StiTestSub2.new.save
     DB.sqls.should == ["INSERT INTO sti_tests (kind) VALUES ('StiTest')", "SELECT * FROM sti_tests WHERE (id = 10) LIMIT 1", "INSERT INTO sti_tests (kind) VALUES ('StiTestSub1')", "SELECT * FROM sti_tests WHERE ((sti_tests.kind IN ('StiTestSub1')) AND (id = 10)) LIMIT 1", "INSERT INTO sti_tests (kind) VALUES ('StiTestSub2')", "SELECT * FROM sti_tests WHERE ((sti_tests.kind IN ('StiTestSub2')) AND (id = 10)) LIMIT 1"]
   end
 
-  it "should have the before_create hook not override an existing value" do
+  it "should handle validations on the type column field" do
+    o = StiTestSub1.new
+    def o.validate
+      errors.add(:kind, 'not present') unless kind
+    end
+    o.valid?.should == true
+  end
+
+  it "should set type column field even if validations are skipped" do
+    StiTestSub1.new.save(:validate=>false)
+    DB.sqls.should == ["INSERT INTO sti_tests (kind) VALUES ('StiTestSub1')", "SELECT * FROM sti_tests WHERE ((sti_tests.kind IN ('StiTestSub1')) AND (id = 10)) LIMIT 1"]
+  end
+
+  it "should override an existing value in the class name field" do
     StiTest.create(:kind=>'StiTestSub1')
     DB.sqls.should == ["INSERT INTO sti_tests (kind) VALUES ('StiTestSub1')", "SELECT * FROM sti_tests WHERE (id = 10) LIMIT 1"]
   end
 
-  it "should have the before_create hook handle columns with the same name as existing method names" do
+  it "should handle type column with the same name as existing method names" do
     StiTest.plugin :single_table_inheritance, :type
     StiTest.columns :id, :type
     StiTest.create
@@ -107,6 +120,29 @@ describe Sequel::Model, "single table inheritance plugin" do
     StiTestSub1A.dataset.sql.should == "SELECT * FROM sti_tests WHERE (sti_tests.kind IN ('StiTestSub1A', 'StiTestSub1B'))"
     StiTestSub1B.dataset.sql.should == "SELECT * FROM sti_tests WHERE (sti_tests.kind IN ('StiTestSub1B'))"
   end
+  
+  it "should work correctly with the :caching plugin" do
+    cache_class = Class.new(Hash) do
+      attr_accessor :ttl
+      def set(k, v, ttl); self[k] = v; @ttl = ttl; end
+      def get(k); self[k]; end
+    end
+    cache = cache_class.new
+
+    StiTest.plugin :caching, cache
+    def StiTest.cache_key_prefix; "stitest" end
+    c2 = Class.new StiTest
+    c2.cache_key(:id).should == StiTest.cache_key(:id)
+
+    obj2 = c2.new
+    obj2.values[:x] = 2
+    obj2.save
+    c2[obj2.id]
+    c2.cache_get_pk(obj2.id).values.should == StiTest.cache_get_pk(obj2.id).values
+    obj2.save
+    c2.cache_get_pk(obj2.id).should == nil
+    StiTest.cache_get_pk(obj2.id).should == nil
+  end
 
   describe "with custom options" do
     before do
@@ -236,6 +272,5 @@ describe Sequel::Model, "single table inheritance plugin" do
       StiTest3.create.kind.should == 'stitest3'
       StiTest4.create.kind.should == 'stitest4'
     end
-
   end
 end
