@@ -1,10 +1,17 @@
 require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 
+model_class = proc do |klass, &block|
+  c = Class.new(klass)
+  c.plugin :validation_class_methods
+  c.class_eval(&block) if block
+  c
+end
+
 describe Sequel::Model do
   before do
-    @c = Class.new(Sequel::Model) do
+    @c = model_class.call Sequel::Model do
       def self.validates_coolness_of(attr)
-        validates_each(attr) {|o, a, v| o.errors[a] << 'is not cool' if v != :cool}
+        validates_each(attr) {|o, a, v| o.errors.add(a, 'is not cool') if v != :cool}
       end
     end
   end
@@ -34,7 +41,7 @@ describe Sequel::Model do
   end
 
   specify "should acccept validation definitions using validates_each" do
-    @c.validates_each(:xx, :yy) {|o, a, v| o.errors[a] << 'too low' if v < 50}
+    @c.validates_each(:xx, :yy) {|o, a, v| o.errors.add(a, 'too low') if v < 50}
     o = @c.new
     o.should_receive(:xx).once.and_return(40)
     o.should_receive(:yy).once.and_return(60)
@@ -98,8 +105,8 @@ describe Sequel::Model do
   end
   
   specify "should overwrite existing validation with the same tag and attribute" do
-    @c.validates_each(:xx, :xx, :tag=>:low) {|o, a, v| o.xxx; o.errors[a] << 'too low' if v < 50}
-    @c.validates_each(:yy, :yy) {|o, a, v| o.yyy; o.errors[a] << 'too low' if v < 50}
+    @c.validates_each(:xx, :xx, :tag=>:low) {|o, a, v| o.xxx; o.errors.add(a, 'too low') if v < 50}
+    @c.validates_each(:yy, :yy) {|o, a, v| o.yyy; o.errors.add(a, 'too low') if v < 50}
     @c.validates_presence_of(:zz, :zz)
     @c.validates_length_of(:aa, :aa, :tag=>:blah)
     o = @c.new
@@ -131,15 +138,22 @@ describe Sequel::Model do
     o.valid?.should == false
     o.errors.full_messages.should == ['blah is not cool']
   end
+
+  specify "should have the validates block have appropriate respond_to?" do
+    c = nil
+    @c.validates{c = respond_to?(:foo)}
+    c.should == false
+    @c.validates{c = respond_to?(:length_of)}
+    c.should == true
+  end if RUBY_VERSION >= '1.9'
 end
 
 describe Sequel::Model do
   before do
-    @c = Class.new(Sequel::Model)
-    @c.class_eval do
+    @c = model_class.call Sequel::Model do
       columns :score
       validates_each :score do |o, a, v|
-        o.errors[a] << 'too low' if v < 87
+        o.errors.add(a, 'too low') if v < 87
       end
     end
     
@@ -161,17 +175,17 @@ describe Sequel::Model do
     @o.score = 86
     @o.should_not be_valid
     @o.errors[:score].should == ['too low']
-    @o.errors[:blah].should be_empty
+    @o.errors.on(:blah).should be_nil
   end
 end
 
-describe Sequel::Plugins::ValidationClassMethods::ClassMethods::Generator do
+describe "Sequel::Plugins::ValidationClassMethods::ClassMethods::Generator" do
   before do
-    $testit = nil
+    @testit = testit = []
     
-    @c = Class.new(Sequel::Model) do
-      def self.validates_blah
-        $testit = 1324
+    @c = model_class.call Sequel::Model do
+      (class << self; self end).send(:define_method, :validates_blah) do
+        testit << 1324
       end
     end
   end
@@ -180,13 +194,13 @@ describe Sequel::Plugins::ValidationClassMethods::ClassMethods::Generator do
     @c.validates do
       blah
     end
-    $testit.should == 1324
+    @testit.should == [1324]
   end
 end
 
 describe Sequel::Model do
   before do
-    @c = Class.new(Sequel::Model) do
+    @c = model_class.call Sequel::Model do
       columns :value
       
       def self.filter(*args)
@@ -572,15 +586,14 @@ describe Sequel::Model do
   specify "should validate uniqueness_of with allow_missing => true" do
     @c.validates_uniqueness_of :value, :allow_missing => true
     @m.should be_valid
-    @m.value = nil
+    @m.value = 1
     @m.should_not be_valid
   end
 end
 
 describe "Superclass validations" do
   before do
-    @c1 = Class.new(Sequel::Model)
-    @c1.class_eval do
+    @c1 = model_class.call Sequel::Model do
       columns :value
       validates_length_of :value, :minimum => 5
     end
@@ -603,33 +616,38 @@ describe "Superclass validations" do
     o.errors.full_messages.should == ['value is too short', 'value is invalid']
 
     o.value = 'abcde'
-    o.valid?.should be_true
+    o.valid?.should == true
   end
   
+  specify "should have skip_superclass_validations? return whether superclass validations were skipped" do
+    @c2.skip_superclass_validations?.should == nil
+    @c2.skip_superclass_validations
+    @c2.skip_superclass_validations?.should == true
+  end
+
   specify "should be skipped if skip_superclass_validations is called" do
     @c2.skip_superclass_validations
 
     o = @c2.new
     o.value = 'ab'
-    o.valid?.should be_true
+    o.valid?.should == true
 
     o.value = '12'
     o.valid?.should == false
     o.errors.full_messages.should == ['value is invalid']
 
     o.value = 'abcde'
-    o.valid?.should be_true
+    o.valid?.should == true
   end
 end
 
 describe ".validates with block" do
   specify "should support calling .each" do
-    @c = Class.new(Sequel::Model)
-    @c.class_eval do
+    @c = model_class.call Sequel::Model do
       columns :vvv
       validates do
         each :vvv do |o, a, v|
-          o.errors[a] << "is less than zero" if v.to_i < 0
+          o.errors.add(a, "is less than zero") if v.to_i < 0
         end
       end
     end
@@ -646,24 +664,36 @@ describe Sequel::Model, "Validations" do
 
   before(:all) do
     class ::Person < Sequel::Model
+      plugin :validation_class_methods
       columns :id,:name,:first_name,:last_name,:middle_name,:initials,:age, :terms
     end
 
     class ::Smurf < Person
     end
+
+    class ::Can < Sequel::Model
+      plugin :validation_class_methods
+      columns :id, :name
+    end
     
     class ::Cow < Sequel::Model
+      plugin :validation_class_methods
       columns :id, :name, :got_milk
     end
 
     class ::User < Sequel::Model
+      plugin :validation_class_methods
       columns :id, :username, :password
     end
     
     class ::Address < Sequel::Model
+      plugin :validation_class_methods
       columns :id, :zip_code
     end
   end
+  after(:all) do
+    [:Person, :Smurf, :Cow, :User, :Address].each{|c| Object.send(:remove_const, c)}
+  end
   
   it "should validate the acceptance of a column" do
     class ::Cow < Sequel::Model
@@ -703,9 +733,9 @@ describe Sequel::Model, "Validations" do
     end
 
     @person = Person.new :first_name => "Lancelot99"
-    @person.valid?.should be_false
+    @person.valid?.should == false
     @person = Person.new :first_name => "Anita"
-    @person.valid?.should be_true
+    @person.valid?.should == true
   end
   
   it "should validate length of column" do
@@ -740,42 +770,31 @@ describe Sequel::Model, "Validations" do
     @person.should be_valid
   end
   
-  it "should validate that a column doesn't have a string value" do
-    p = Class.new(Sequel::Model)
-    p.class_eval do
-      columns :age, :price, :confirmed
+  it "should validate that a column has the correct type for the schema column" do
+    p = model_class.call Sequel::Model do
+      columns :age, :d
       self.raise_on_typecast_failure = false
-      validates_not_string :age
-      validates_not_string :confirmed
-      validates_not_string :price, :message=>'is not valid'
-      @db_schema = {:age=>{:type=>:integer}}
+      validates_schema_type :age
+      validates_schema_type :d, :message=>'is a bad choice'
+      @db_schema = {:age=>{:type=>:integer}, :d=>{:type=>:date}}
     end
     
     @person = p.new
     @person.should be_valid
 
-    @person.confirmed = 't'
-    @person.should_not be_valid
-    @person.errors.full_messages.should == ['confirmed is a string']
-    @person.confirmed = true
-    @person.should be_valid
-
     @person.age = 'a'
     @person.should_not be_valid
     @person.errors.full_messages.should == ['age is not a valid integer']
-    @person.db_schema[:age][:type] = :datetime
-    @person.should_not be_valid
-    @person.errors.full_messages.should == ['age is not a valid datetime']
-    @person.age = 20
+    @person.age = 1
     @person.should be_valid
 
-    @person.price = 'a'
+    @person.d = 'a'
     @person.should_not be_valid
-    @person.errors.full_messages.should == ['price is not valid']
-    @person.price = 20
+    @person.errors.full_messages.should == ['d is a bad choice']
+    @person.d = Date.today
     @person.should be_valid
   end
-  
+
   it "should validate numericality of column" do
     class ::Person < Sequel::Model
       validations.clear
@@ -813,11 +832,11 @@ describe Sequel::Model, "Validations" do
     end
     User.dataset._fetch = proc do |sql|
       case sql
-      when /COUNT.*username = '0records'/
+      when /count.*username = '0records'/
         {:v => 0}
-      when /COUNT.*username = '2records'/
+      when /count.*username = '2records'/
         {:v => 2}
-      when /COUNT.*username = '1record'/
+      when /count.*username = '1record'/
         {:v => 1}
       when /username = '1record'/
         {:id => 3, :username => "1record", :password => "test"}
@@ -843,6 +862,12 @@ describe Sequel::Model, "Validations" do
     @user = User.new(:username => "0records", :password => "anothertest")
     @user.should be_valid
     @user.errors.full_messages.should == []
+
+    User.db.sqls
+    @user = User.new(:password => "anothertest")
+    @user.should be_valid
+    @user.errors.full_messages.should == []
+    User.db.sqls.should == []
   end
   
   it "should validate the uniqueness of multiple columns" do
@@ -854,11 +879,11 @@ describe Sequel::Model, "Validations" do
     end
     User.dataset._fetch = proc do |sql|
       case sql
-      when /COUNT.*username = '0records'/
+      when /count.*username = '0records'/
         {:v => 0}
-      when /COUNT.*username = '2records'/
+      when /count.*username = '2records'/
         {:v => 2}
-      when /COUNT.*username = '1record'/
+      when /count.*username = '1record'/
         {:v => 1}
       when /username = '1record'/
         if sql =~ /password = 'anothertest'/
@@ -892,6 +917,18 @@ describe Sequel::Model, "Validations" do
     @user = User.new(:username => "0records", :password => "anothertest")
     @user.should be_valid
     @user.errors.full_messages.should == []
+
+    User.db.sqls
+    @user = User.new(:password => "anothertest")
+    @user.should be_valid
+    @user.errors.full_messages.should == []
+    @user = User.new(:username => "0records")
+    @user.should be_valid
+    @user.errors.full_messages.should == []
+    @user = User.new
+    @user.should be_valid
+    @user.errors.full_messages.should == []
+    User.db.sqls.should == []
   end
   
   it "should have a validates block that contains multiple validations" do
@@ -906,10 +943,10 @@ describe Sequel::Model, "Validations" do
     Person.validations[:first_name].size.should == 2
     
     @person = Person.new :first_name => "Lancelot99"
-    @person.valid?.should be_false
+    @person.valid?.should == false
     
     @person2 = Person.new :first_name => "Wayne"
-    @person2.valid?.should be_true
+    @person2.valid?.should == true
   end
 
   it "should allow 'longhand' validations direcly within the model." do
@@ -941,8 +978,6 @@ describe Sequel::Model, "Validations" do
 
   it "should validate correctly instances initialized with string keys" do
     class ::Can < Sequel::Model
-      columns :id, :name
-      
       validates_length_of :name, :minimum => 4
     end
     
@@ -954,35 +989,34 @@ end
 
 describe "Model#save" do
   before do
-    @c = Class.new(Sequel::Model(:people))
-    @c.class_eval do
+    @c = model_class.call Sequel::Model(:people) do
       columns :id, :x
 
       validates_each :x do |o, a, v|
-        o.errors[a] << 'blah' unless v == 7
+        o.errors.add(a, 'blah') unless v == 7
       end
     end
     @m = @c.load(:id => 4, :x=>6)
-    MODEL_DB.reset
+    DB.reset
   end
 
   specify "should save only if validations pass" do
     @m.raise_on_save_failure = false
     @m.should_not be_valid
     @m.save
-    MODEL_DB.sqls.should be_empty
+    DB.sqls.should be_empty
     
     @m.x = 7
     @m.should be_valid
-    @m.save.should_not be_false
-    MODEL_DB.sqls.should == ['UPDATE people SET x = 7 WHERE (id = 4)']
+    @m.save.should_not == false
+    DB.sqls.should == ['UPDATE people SET x = 7 WHERE (id = 4)']
   end
   
   specify "should skip validations if the :validate=>false option is used" do
     @m.raise_on_save_failure = false
     @m.should_not be_valid
     @m.save(:validate=>false)
-    MODEL_DB.sqls.should == ['UPDATE people SET x = 6 WHERE (id = 4)']
+    DB.sqls.should == ['UPDATE people SET x = 6 WHERE (id = 4)']
   end
     
   specify "should raise error if validations fail and raise_on_save_faiure is true" do
