@@ -142,6 +142,13 @@ describe "Database schema parser" do
     DB.create_table!(:items){FalseClass :number}
     DB.schema(:items).first.last[:type].should == :boolean
   end
+
+  specify "should parse maximum length for string columns" do
+    DB.create_table!(:items){String :a, :size=>4}
+    DB.schema(:items).first.last[:max_length].should == 4
+    DB.create_table!(:items){String :a, :fixed=>true, :size=>3}
+    DB.schema(:items).first.last[:max_length].should == 3
+  end
 end if DB.supports_schema_parsing?
 
 describe "Database index parsing" do
@@ -357,7 +364,7 @@ describe "Database schema modifiers" do
     end
   end
 
-  specify "should create temporary tables without raising an exception" do
+  cspecify "should create temporary tables without raising an exception", :fdbsql do
     @db.create_table!(:items_temp, :temp=>true){Integer :number}
   end
 
@@ -413,7 +420,7 @@ describe "Database schema modifiers" do
     @ds.insert([10])
   end
 
-  specify "should be able to specify constraint names for column constraints" do
+  cspecify "should be able to specify constraint names for column constraints", :fdbsql do
     @db.create_table!(:items2){primary_key :id, :primary_key_constraint_name=>:foo_pk}
     @db.create_table!(:items){foreign_key :id, :items2, :unique=>true, :foreign_key_constraint_name => :foo_fk, :unique_constraint_name => :foo_uk, :null=>false}
     @db.alter_table(:items){drop_constraint :foo_fk, :type=>:foreign_key; drop_constraint :foo_uk, :type=>:unique}
@@ -674,7 +681,7 @@ describe "Database schema modifiers" do
     @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:id]
   end
 
-  cspecify "should work correctly with many operations in a single alter_table call", [:jdbc, :db2], [:db2] do
+  cspecify "should work correctly with many operations in a single alter_table call", [:jdbc, :db2], [:db2], :fdbsql do
     @db.create_table!(:items) do
       primary_key :id
       String :name2
